@@ -1145,7 +1145,9 @@ public:
 
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e6;
+    double time_db = 0.;
+    if(gpGuiProcessor && gpGuiProcessor->cpu)
+        time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e6;
     snprintf(buf,size, "%19.2f us",time_db);
   }
 };
@@ -1162,7 +1164,9 @@ public:
 
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e3;
+    double time_db = 0.;
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
+        time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e3;
     snprintf(buf,size, "%19.3f ms",time_db);
   }
 };
@@ -1179,7 +1183,9 @@ public:
 
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
+    double time_db = 0.;
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
+       time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
     snprintf(buf,size, "%19.3f Sec",time_db);
   }
 };
@@ -1196,14 +1202,16 @@ public:
 
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
-    double v=time_db;
+    double time_db = 0.;
+    if(gpGuiProcessor && gpGuiProcessor->cpu)
+        time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
+    double v=time_db + 0.005;	// round msec
     int hh=(int)(v/3600),mm,ss,cc;
     v-=hh*3600.0;
     mm=(int)(v/60);
     v-=mm*60.0;
     ss=(int)v;
-    cc=(int)(v*100.0+0.5);
+    cc=(int)((v-ss)*100.0);
     snprintf(buf,size,"    %02d:%02d:%02d.%02d",hh,mm,ss,cc);
   }
 };
