@@ -1,22 +1,22 @@
 /*
    Copyright (C) 2006 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim_modules library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #ifndef __ttl_H__
 #define __ttl_H__
@@ -40,7 +40,9 @@ namespace TTL {
     ~TTLbase();
 
     virtual void setClock(bool) {}
+    virtual void setStrobe(bool) {}
     virtual void setEnable(bool) {}
+    virtual void setReset(bool) {}
     //virtual void create_iopin_map();
     virtual void update_state()=0;
     //GtkWidget *create_pixmap(char **pixmap_data);
@@ -53,7 +55,9 @@ namespace TTL {
   };
 
   class Clock;
+  class Strobe;
   class Enable;
+  class Reset;
 
   class TTL377 : public TTLbase
   {
@@ -73,6 +77,31 @@ namespace TTL {
     IO_bi_directional **m_Q;
   };
 
+  class TTL595 : public TTLbase
+  {
+  public:
+
+    TTL595(const char *_name);
+    static Module *construct(const char *new_name=NULL);
+
+    virtual void create_iopin_map();
+    virtual void setClock(bool);
+    virtual void setStrobe(bool);
+    virtual void setEnable(bool);
+    virtual void setReset(bool);
+    virtual void update_state();
+  protected:
+    bool    m_bStrobe;
+    Clock  *m_clock;
+    Strobe *m_strobe;
+    Reset  *m_reset;
+    Enable *m_enable;
+    IOPIN  *m_Ds;
+    IOPIN  *m_Qs;
+    IO_bi_directional **m_Q;
+    unsigned short sreg;
+  };
+
 } // end of namespace TTL
 
 #endif // __ttl_H__
