@@ -1,23 +1,23 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2006,2009 Roy R Rankin
+   Copyright (C) 2006,2009,2010 Roy R Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <stdio.h>
 #include <iostream>
@@ -27,7 +27,7 @@ Boston, MA 02111-1307, USA.  */
 #include "../config.h"
 #include "14bit-tmrs.h"
 #include "stimuli.h"
-#include "p16x7x.h"
+#include "a2dconverter.h"
 
 //
 // 14bit-tmrs.cc
@@ -191,7 +191,8 @@ class CCPSignalSource : public SignalControl
 {
 public:
   CCPSignalSource(CCPCON *_ccp)
-    : m_ccp(_ccp)
+    : m_ccp(_ccp),
+    state('?')
   {
     assert(m_ccp);
   }
@@ -199,16 +200,15 @@ public:
   {
   }
 
-  virtual char getState()
-  {
-    return m_ccp->getState();
-  }
+  void setState(char m_state) { state = m_state; }
+  virtual char getState() { return state; }
   virtual void release() 
   {
     delete this;
   }
 private:
   CCPCON *m_ccp;
+  char state;
 };
 
 //--------------------------------------------------
@@ -236,45 +236,93 @@ private:
   CCPCON *m_ccp;
 };
 
+class Tristate : public SignalControl
+{
+public:
+  Tristate() { }
+  ~Tristate() { }
+  char getState() { return '1'; }	// set port to high impedance by setting it to input
+  void release() { delete this; }
+};
 //--------------------------------------------------
 // CCPCON
 //--------------------------------------------------
 CCPCON::CCPCON(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    m_PinModule(0),
-    m_source(0),
+    pstrcon(0),
+    pwm1con(0),
+    eccpas(0),
+    bit_mask(0x3f),
+    m_sink(0),
+    m_tristate(0),
     m_bInputEnabled(false),
     m_bOutputEnabled(false),
     m_cOutputState('?'),
-    edges(0),
+    edges(0), bridge_shutdown(false),
     ccprl(0), pir(0), tmr2(0), adcon0(0)
 {
+	m_PinModule[0] = 0;
 }
 
-CCPCON::~CCPCON()
+CCPCON::~CCPCON() 
+{ 
+}
+
+// EPWM has four outputs PWM 1
+void CCPCON::setIOpin(PinModule *p1, PinModule *p2, PinModule *p3, PinModule *p4)
 {
-  //delete m_sink;
-  //delete m_source;
-}
+  Dprintf(("%s::setIOpin %s\n", name().c_str(), (p1 && &(p1->getPin())) ? p1->getPin().name().c_str():"unknown"));
+  if (!&(p1->getPin()))
+  {
+	Dprintf(("FIXME %s::setIOpin called where p1 has unassigned pin\n", name().c_str()));
+  }
 
-void CCPCON::setIOpin(PinModule *new_PinModule)
-{
-  Dprintf(("CCPCON::setIOpin\n"));
-
-  m_PinModule = new_PinModule;
-
-  m_sink = new CCPSignalSink(this);
-  m_PinModule->addSink(m_sink);
-
-  m_source = new CCPSignalSource(this);
+  if (p1 && &(p1->getPin()))
+  {
+    if (m_PinModule[0])
+    {
+	if (m_PinModule[0] != p1)
+	fprintf(stderr, "FIXME %s::setIOpin called for port %s then %s %p %p\n", 
+		name().c_str(), 
+		m_PinModule[0]->getPin().name().c_str(), 
+		p1->getPin().name().c_str() , m_PinModule[0], p1
+	);
+    }
+    else
+    {
+        m_PinModule[0] = p1;
+        m_sink = new CCPSignalSink(this);
+	m_tristate = new Tristate();
+        m_source[0] = new CCPSignalSource(this);
+        p1->addSink(m_sink);
+    }
+  }
+  m_PinModule[1] = m_PinModule[2] = m_PinModule[3] = 0;
+  m_source[1] = m_source[2] = m_source[3] = 0;
+  if (p2)
+  {
+    m_PinModule[1] = p2;
+    m_source[1] = new CCPSignalSource(this);
+  }
+  if (p3)
+  {
+    m_PinModule[2] = p3;
+    m_source[2] = new CCPSignalSource(this);
+  }
+  if (p4)
+  {
+    m_PinModule[3] = p4;
+    m_source[3] = new CCPSignalSource(this);
+  }
 
 }
 
-void CCPCON::setCrosslinks(CCPRL *_ccprl, PIR *_pir, TMR2 *_tmr2)
+void CCPCON::setCrosslinks(CCPRL *_ccprl, PIR *_pir, TMR2 *_tmr2, ECCPAS *_eccpas)
 {
   ccprl = _ccprl;
   pir = _pir;
   tmr2 = _tmr2;
+  eccpas = _eccpas;
 }
 
 void CCPCON::setADCON(ADCON0 *_adcon0)
@@ -289,7 +337,7 @@ char CCPCON::getState()
 
 void CCPCON::new_edge(unsigned int level)
 {
-  Dprintf(("CCPCON::new_edge() level=%d\n",level));
+  Dprintf(("%s::new_edge() level=%d\n",name().c_str(), level));
 
   switch(value.get() & (CCPM3 | CCPM2 | CCPM1 | CCPM0))
     {
@@ -353,7 +401,7 @@ void CCPCON::new_edge(unsigned int level)
 void CCPCON::compare_match()
 {
 
-  Dprintf(("CCPCON::compare_match()\n"));
+  Dprintf(("%s::compare_match()\n", name().c_str()));
 
   switch(value.get() & (CCPM3 | CCPM2 | CCPM1 | CCPM0))
     {
@@ -373,7 +421,8 @@ void CCPCON::compare_match()
 
     case COM_SET_OUT:
       m_cOutputState = '1';
-      m_PinModule->updatePinModule();
+      m_source[0]->setState('1');
+      m_PinModule[0]->updatePinModule();
       if (pir)
 	pir->set_ccpif();
       Dprintf(("-- CCPCON setting compare output to 1\n"));
@@ -381,7 +430,8 @@ void CCPCON::compare_match()
 
     case COM_CLEAR_OUT:
       m_cOutputState = '0';
-      m_PinModule->updatePinModule();
+      m_source[0]->setState('0');
+      m_PinModule[0]->updatePinModule();
       if (pir)
 	pir->set_ccpif();
       Dprintf(("-- CCPCON setting compare output to 0\n"));
@@ -410,40 +460,310 @@ void CCPCON::compare_match()
     case PWM3:
       //cout << "CCPCON is set up as an output\n";
       return;
+
     }
 }
 
+// handle dead-band delay in half-bridge mode
+void CCPCON::callback()
+{
+
+    if(delay_source0)
+    {
+       	m_source[0]->setState('1');
+    	m_PinModule[0]->updatePinModule();
+        delay_source0 = false;
+    }
+    if(delay_source1)
+    {
+       	m_source[1]->setState('1');
+    	m_PinModule[1]->updatePinModule();
+        delay_source1 = false;
+    }
+}
 void CCPCON::pwm_match(int level)
 {
-  Dprintf(("CCPCON::pwm_match()\n"));
+  unsigned int new_value = value.get();
+  Dprintf(("%s::pwm_match() level=%d\n", name().c_str(), level));
 
-  if( (value.get() & PWM0) == PWM0) {
 
-    m_cOutputState = level ? '1' : '0';
+  // if the level is 1, then tmr2 == pr2 and the pwm cycle
+  // is starting over. In which case, we need to update the duty
+  // cycle by reading ccprl and the ccp X & Y and caching them
+  // in ccprh's pwm slave register.
+  if(level == 1) {
+      // Auto shutdown comes off at start of PWM if ECCPASE clear
+      if (bridge_shutdown && (!eccpas || !(eccpas->get_value() & ECCPAS::ECCPASE)))
+      {
+          Dprintf(("bridge_shutdown=%d eccpas=%p ECCPAS=%x\n", bridge_shutdown, eccpas, 
+		eccpas ? eccpas->get_value() & ECCPAS::ECCPASE: 0));
+	  for(int i = 0; i < 4; i++)
+	  {
+            if(m_PinModule[i])
+	    {
+		 m_PinModule[i]->setControl(0); //restore defailt pin direction
+        	 m_PinModule[i]->updatePinModule();
+	    }
+	  }
+	  bridge_shutdown = false;
+      }
 
-    // if the level is 'high', then tmr2 == pr2 and the pwm cycle
-    // is starting over. In which case, we need to update the duty
-    // cycle by reading ccprl and the ccp X & Y and caching them
-    // in ccprh's pwm slave register.
-
-    if(level) {
       ccprl->ccprh->pwm_value = ((value.get()>>4) & 3) | 4*ccprl->value.get();
       tmr2->pwm_dc(ccprl->ccprh->pwm_value, address);
       ccprl->ccprh->put_value(ccprl->value.get());
-      if (!ccprl->ccprh->pwm_value)  // if duty cycle == 0 output stays low
-	m_cOutputState = '0';
-    }
-    m_PinModule->updatePinModule();
+  }
+  if( !pwm1con) { // simple PWM
 
-    //cout << "iopin should change\n";
-  }  else
-    cout << "not pwm mode. bug?\n";
+    if (bridge_shutdown == false) // some processors use shutdown and simple PWM
+    {
+        m_cOutputState = level ? '1' : '0';
+        m_source[0]->setState(level ? '1' : '0');
+        m_PinModule[0]->setSource(m_source[0]);
+    
+    
+        if(level && !ccprl->ccprh->pwm_value)  // if duty cycle == 0 output stays low
+              m_source[0]->setState('0');
+
+        m_PinModule[0]->updatePinModule();
+    
+        //cout << "iopin should change\n";
+    }
+  }  
+  else	// EPWM
+  {
+
+
+      if (!bridge_shutdown)
+      	drive_bridge(level, new_value);
+  }
+}
+//
+//  Drive PWM bridge
+//
+void CCPCON::drive_bridge(int level, int new_value)
+{
+      unsigned int pstrcon_value;
+      // pstrcon allows port steering for "single output"
+      // if it is not defined, just use the first port
+      if (pstrcon)
+	  pstrcon_value = pstrcon->value.get();
+      else
+	  pstrcon_value = 1;
+      int pwm_width;
+      int delay = pwm1con->value.get() & ~PWM1CON::PRSEN;
+
+      bool active_high[4];
+      switch (new_value & (CCPM3|CCPM2|CCPM1|CCPM0))
+      {
+        case PWM0:	//P1A, P1C, P1B, P1D active high
+	active_high[0] = true;
+	active_high[1] = true;
+	active_high[2] = true;
+	active_high[3] = true;
+    	break;
+    
+        case PWM1:	// P1A P1C active high P1B P1D active low
+	active_high[0] = true;
+	active_high[1] = false;
+	active_high[2] = true;
+	active_high[3] = false;
+    	break;
+    
+        case PWM2: 	// P1A P1C active low P1B P1D active high 
+	active_high[0] = false;
+	active_high[1] = true;
+	active_high[2] = false;
+	active_high[3] = true;
+    	break;
+    
+        case PWM3:	// //P1A, P1C, P1B, P1D active low
+	active_high[0] = false;
+	active_high[1] = false;
+	active_high[2] = false;
+	active_high[3] = false;
+    	break;
+    
+        default:
+            cout << "not pwm mode. bug?\n";
+    	return;
+    	break;
+      }
+      switch((new_value & (P1M1|P1M0))>>6) // ECCP bridge mode
+      {
+	    case 0:	// Single
+		Dprintf(("Single bridge pstrcon=0x%x\n", pstrcon_value));
+		for (int i = 0; i <4; i++)
+		{
+		    if (pstrcon_value & (1<<i))
+		    {
+			m_PinModule[i]->setSource(m_source[i]);
+			// follow level except where duty cycle = 0
+			if (level && ccprl->ccprh->pwm_value)
+          		    m_source[i]->setState(active_high[i]?'1':'0');
+			else
+          		    m_source[i]->setState(active_high[i]?'0':'1');
+    			m_PinModule[i]->updatePinModule();
+		    }
+		    else if (m_PinModule[i])
+			m_PinModule[i]->setSource(0);
+		}
+		break;
+	
+	    case 2:	// Half-Bridge
+		Dprintf(("half-bridge\n"));
+		m_PinModule[0]->setSource(m_source[0]);
+		m_PinModule[1]->setSource(m_source[1]);
+		m_PinModule[2]->setSource(0);
+		m_PinModule[3]->setSource(0);
+		delay_source0 = false;
+		delay_source1 = false;
+		// FIXME need to add deadband
+		// follow level except where duty cycle = 0
+		pwm_width = level ? 
+			ccprl->ccprh->pwm_value : 
+			((tmr2->pr2->value.get()+1)*4)-ccprl->ccprh->pwm_value;
+		if (!(level^active_high[0]) && ccprl->ccprh->pwm_value)
+		{
+		    // No delay, change state
+		    if (delay == 0)
+       		    	m_source[0]->setState('1');
+		    else if (delay < pwm_width) // there is a delay
+		    {
+			future_cycle = get_cycles().get() + delay;
+          		get_cycles().set_break(future_cycle, this);
+			delay_source0 = true;
+		    }
+		}
+		else
+		{
+       		    m_source[0]->setState('0');
+		}
+		if (!(level^active_high[1]) && ccprl->ccprh->pwm_value)
+		{
+       		    m_source[1]->setState('0');
+		}
+		else
+		{
+		    // No delay, change state
+		    if (delay == 0)
+       		    	m_source[1]->setState('1');
+		    else if (delay < pwm_width) // there is a delay
+		    {
+			future_cycle = get_cycles().get() + delay;
+          		get_cycles().set_break(future_cycle, this);
+			delay_source1 = true;
+		    }
+		}
+    		m_PinModule[0]->updatePinModule();
+    		m_PinModule[1]->updatePinModule();
+		break;
+	
+	    case 1:	// Full bidge Forward
+		Dprintf(("full-bridge, forward\n"));
+		m_PinModule[0]->setSource(m_source[0]);
+		m_PinModule[1]->setSource(m_source[1]);
+		m_PinModule[2]->setSource(m_source[2]);
+		m_PinModule[3]->setSource(m_source[3]);
+		// P1D toggles
+		if (level && ccprl->ccprh->pwm_value)
+          	    m_source[3]->setState(active_high[3]?'1':'0');
+		else
+          	    m_source[3]->setState(active_high[3]?'0':'1');
+		// P1A High (if active high)
+          	m_source[0]->setState(active_high[0]?'1':'0');
+		// P1B, P1C low (if active high)
+          	m_source[1]->setState(active_high[1]?'0':'1');
+          	m_source[2]->setState(active_high[2]?'0':'1');
+    		m_PinModule[0]->updatePinModule();
+    		m_PinModule[1]->updatePinModule();
+    		m_PinModule[2]->updatePinModule();
+    		m_PinModule[3]->updatePinModule();
+		break;
+	
+	    case 3:	// Full bridge reverse
+		Dprintf(("full-bridge reverse\n"));
+		m_PinModule[0]->setSource(m_source[0]);
+		m_PinModule[1]->setSource(m_source[1]);
+		m_PinModule[2]->setSource(m_source[2]);
+		m_PinModule[3]->setSource(m_source[3]);
+		// P1B toggles
+		if (level && ccprl->ccprh->pwm_value)
+          	    m_source[1]->setState(active_high[1]?'1':'0');
+		else
+          	    m_source[1]->setState(active_high[1]?'0':'1');
+		// P1C High (if active high)
+          	m_source[2]->setState(active_high[2]?'1':'0');
+		// P1A, P1D low (if active high)
+          	m_source[0]->setState(active_high[0]?'0':'1');
+          	m_source[3]->setState(active_high[3]?'0':'1');
+    		m_PinModule[0]->updatePinModule();
+    		m_PinModule[1]->updatePinModule();
+    		m_PinModule[2]->updatePinModule();
+    		m_PinModule[3]->updatePinModule();
+		break;
+
+	    default:
+		printf("%s::pwm_match impossible ECCP bridge mode\n", name().c_str());
+		break;
+       }
+	
+}
+//
+// Set PWM bridge into shutdown mode
+//
+void CCPCON::shutdown_bridge(int eccpas)
+{
+    bridge_shutdown = true;
+
+    Dprintf(("eccpas=0x%x\n", eccpas));
+
+    switch(eccpas & (ECCPAS::PSSBD0 | ECCPAS::PSSBD1))
+    {
+    case 0:	// B D output 0
+	if (m_source[1]) m_source[1]->setState('0');
+	if (m_source[3]) m_source[3]->setState('0');
+	break;
+
+    case 1:	// B, D output 1
+	if (m_source[1]) m_source[1]->setState('1');
+	if (m_source[3]) m_source[3]->setState('1');
+	break;
+
+    default:	// Tristate B & D
+        if(m_PinModule[1])  m_PinModule[1]->setControl(m_tristate);
+        if(m_PinModule[3])  m_PinModule[3]->setControl(m_tristate);
+	break;
+    }
+    switch(eccpas & ((ECCPAS::PSSAC0 | ECCPAS::PSSAC1) >> 2))
+    {
+    case 0:	// A, C output 0
+	m_source[0]->setState('0');
+	if (m_source[2]) m_source[2]->setState('0');
+	break;
+
+    case 1:	// A, C output 1
+	m_source[0]->setState('1');
+	if (m_source[2]) m_source[2]->setState('1');
+	break;
+
+    default:	// Tristate A & C
+        m_PinModule[0]->setControl(m_tristate);
+        if(m_PinModule[2])  m_PinModule[2]->setControl(m_tristate);
+	break;
+    }
+    m_PinModule[0]->updatePinModule();
+    if (m_PinModule[1]) m_PinModule[1]->updatePinModule();
+    if (m_PinModule[2]) m_PinModule[2]->updatePinModule();
+    if (m_PinModule[3]) m_PinModule[3]->updatePinModule();
 }
 
 void CCPCON::put(unsigned int new_value)
 {
 
-  Dprintf(("CCPCON::put() new_value=0x%x\n",new_value));
+  unsigned int old_value =  value.get();
+  new_value &= bit_mask;
+  
+  Dprintf(("%s::put() new_value=0x%x\n",name().c_str(), new_value));
   trace.raw(write_trace.get() | value.get());
 
   value.put(new_value);
@@ -471,6 +791,7 @@ void CCPCON::put(unsigned int new_value)
 
       // RP - According to 16F87x data sheet section 8.2.1 clearing CCP1CON also clears the latch
       m_cOutputState = '0';
+      m_source[0]->setState('0');
       break;
 
     case CAP_FALLING_EDGE:
@@ -528,19 +849,34 @@ void CCPCON::put(unsigned int new_value)
       tmr2->pwm_dc( ccprl->ccprh->pwm_value, address);
 */
       m_bInputEnabled = false;
-      m_bOutputEnabled = true;
+      m_bOutputEnabled = false;	// this is done in pwm_match
       m_cOutputState = '0';
+      if ((old_value & P1M0) && (new_value & P1M0)) // old and new full-bridge
+      {		// need to adjust timer if P1M1 also changed
+	Dprintf(("full bridge repeat old=0x%x new=%x\n", old_value, new_value));
+      }
+      else
+          pwm_match(2);
+      return;
       break;
 
     }
 
-  if (oldbOutEn != m_bOutputEnabled && m_PinModule) 
-    m_PinModule->setSource(m_bOutputEnabled ? m_source : 0);
+    if (oldbOutEn != m_bOutputEnabled && m_PinModule)
+    {
+	if (m_bOutputEnabled)
+            m_PinModule[0]->setSource(m_source[0]);
+	else
+	{
+            m_PinModule[0]->setSource(0);
+	    m_source[0]->setState('?');
+	}
+    }
 
-  if ((oldbInEn  != m_bInputEnabled  || 
+    if ((oldbInEn  != m_bInputEnabled  ||
        oldbOutEn != m_bOutputEnabled)
-      && m_PinModule)
-    m_PinModule->updatePinModule();
+      && m_PinModule[0])
+        m_PinModule[0]->updatePinModule();
 
 }
 
@@ -814,6 +1150,7 @@ void TMRL::release()
 
 void TMRL::setIOpin(PinModule *extClkSource)
 {
+  Dprintf(("%s::setIOpin %s\n", name().c_str(), extClkSource?extClkSource->getPin().name().c_str():""));
   Dprintf(("TMRL::setIOpin\n"));
 
   if (extClkSource)
@@ -1008,7 +1345,7 @@ void TMRL::on_or_off(int new_state)
 void TMRL::update()
 {
 
-  Dprintf(("TMR1 update 0x%llx\n",get_cycles().get()));
+  Dprintf(("TMR1 update 0x%"PRINTF_GINT64_MODIFIER"x\n",get_cycles().get()));
 
   // The second part of the if will always be true unless TMR1 Gate enable
   // pin has been defined by a call to TMRL::setGatepin()
@@ -1361,6 +1698,7 @@ void PR2::put(unsigned int new_value)
 
   trace.raw(write_trace.get() | value.get());
   //trace.register_write(address,value.get());
+  Dprintf(("PR2:: put %x\n", new_value));
 
   if(value.get() != new_value)
     {
@@ -1413,7 +1751,7 @@ TMR2::TMR2(Processor *pCpu, const char *pName, const char *pDesc)
     prescale(1),
     prescale_counter(0),
     last_cycle(0),
-    pr2(0), pir_set(0), t2con(0), ccp1con(0), ccp2con(0)
+    pr2(0), pir_set(0), t2con(0), ccp1con(0), ccp2con(0), ssp_module(0)
 {
   value.put(0);
   future_cycle = 0;
@@ -1826,6 +2164,7 @@ void TMR2::new_pre_post_scale()
 
 void TMR2::new_pr2(unsigned int new_value)
 {
+  Dprintf(("TMR2::new_pr2 on=%d\n", t2con->get_tmr2on()));
 
   if(t2con->get_tmr2on())
     {
@@ -1936,9 +2275,10 @@ void TMR2::callback()
 
          if (ssp_module)
                ssp_module->tmr2_clock();
-
           if ((ccp1con->value.get() & CCPCON::PWM0) == CCPCON::PWM0)
+	  {
                ccp1con->pwm_match(1);
+	  }
 
           if ((ccp2con->value.get() & CCPCON::PWM0) == CCPCON::PWM0)
                ccp2con->pwm_match(1);
@@ -1985,3 +2325,177 @@ void TMR2_MODULE::initialize(T2CON *t2con_, PR2 *pr2_, TMR2  *tmr2_)
   tmr2  = tmr2_;
 
 }
+
+//--------------------------------------------------
+//
+//--------------------------------------------------
+
+class INT_SignalSink : public SignalSink
+{
+public:
+  INT_SignalSink(ECCPAS *_eccpas, int _index)
+    : m_eccpas(_eccpas), m_index(_index)
+  {
+    assert(_eccpas);
+  }
+
+  void release() 
+  {
+    delete this;
+  }
+  void setSinkState(char new3State)
+  {
+    m_eccpas->set_trig_state( m_index, new3State=='0' || new3State=='w');
+  }
+private:
+  ECCPAS *m_eccpas;
+  int     m_index;
+};
+
+//--------------------------------------------------
+// ECCPAS
+//--------------------------------------------------
+ECCPAS::ECCPAS(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    pwm1con(0), ccp1con(0), bit_mask(0xff),
+    m_PinModule(0)
+{
+    trig_state[0] = trig_state[1] = trig_state[2] = false;
+}
+
+ECCPAS::~ECCPAS()
+{
+}
+void ECCPAS::link_registers(PWM1CON *_pwm1con, CCPCON *_ccp1con)
+{
+	pwm1con = _pwm1con;
+	ccp1con = _ccp1con;
+}
+void ECCPAS::put(unsigned int new_value)
+{
+  Dprintf(("ECCPAS::put() new_value=0x%x\n",new_value));
+  trace.raw(write_trace.get() | value.get());
+  put_value(new_value);
+}
+void ECCPAS::put_value(unsigned int new_value)
+{
+
+  int old_value = value.get();
+  new_value &= bit_mask;
+
+
+  // Auto-shutdown trigger active
+  //   make sure ECCPASE is set
+  //   if change in shutdown status, drive bridge outputs as per current flags
+  if (shutdown_trigger(new_value))
+  {
+	new_value |= ECCPASE;
+	if ((new_value ^ old_value) &  (ECCPASE|PSSAC1|PSSAC0|PSSBD1|PSSBD0))
+	    ccp1con->shutdown_bridge(new_value);
+  }
+  else // no Auto-shutdown triggers active
+  {
+      if (pwm1con->value.get() & PWM1CON::PRSEN) // clear ECCPASE bit
+	new_value &= ~ ECCPASE;
+  }
+  value.put(new_value);
+}
+// Return true is shutdown trigger is active
+bool ECCPAS::shutdown_trigger(int key)
+{
+
+  if ((key & ECCPAS0) && trig_state[0])
+	return true;
+
+  if ((key & ECCPAS1) && trig_state[1])
+	return true;
+
+  if ((key & ECCPAS2) && trig_state[2])
+	return true;
+
+  return false;
+}
+// connect IO pins to shutdown trigger source
+void ECCPAS::setIOpin(PinModule *p0, PinModule *p1, PinModule *p2)
+{
+    if (p0)
+    {
+        m_PinModule = p0;
+        m_sink = new INT_SignalSink(this, 0);
+        p0->addSink(m_sink);
+    }
+    if (p1)
+    {
+        m_PinModule = p1;
+        m_sink = new INT_SignalSink(this, 1);
+        p1->addSink(m_sink);
+    }
+    if (p2)
+    {
+        m_PinModule = p2;
+        m_sink = new INT_SignalSink(this, 2);
+        p2->addSink(m_sink);
+    }
+}
+
+// set shutdown trigger states
+void ECCPAS::set_trig_state(int index, bool state)
+{
+    if (trig_state[index] != state)
+    {
+	Dprintf(("index=%d state=%d old=%d\n", index, state, trig_state[index]));
+        trig_state[index] = state;
+        put_value(value.get());
+    }
+}
+// Trigger state from comparator 1
+void ECCPAS::c1_output(int state)
+{
+    set_trig_state(0, state);
+}
+// Trigger state from comparator 2
+void ECCPAS::c2_output(int state)
+{
+    set_trig_state(1, state);
+}
+//--------------------------------------------------
+// PWM1CON
+//--------------------------------------------------
+PWM1CON::PWM1CON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+  bit_mask(0xff)
+{
+}
+
+PWM1CON::~PWM1CON()
+{
+}
+void PWM1CON::put(unsigned int new_value)
+{
+
+  new_value &= bit_mask;
+  Dprintf(("PWM1CON::put() new_value=0x%x\n",new_value));
+  trace.raw(write_trace.get() | value.get());
+
+  value.put(new_value);
+}
+//--------------------------------------------------
+// PSTRCON
+//--------------------------------------------------
+PSTRCON::PSTRCON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
+{
+}
+
+PSTRCON::~PSTRCON()
+{
+}
+void PSTRCON::put(unsigned int new_value)
+{
+
+  Dprintf(("PSTRCON::put() new_value=0x%x\n",new_value));
+  new_value &= STRSYNC|STRD|STRC|STRB|STRA;
+  trace.raw(write_trace.get() | value.get());
+
+  value.put(new_value);
+}
