@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <stdio.h>
 #include <iostream>
@@ -25,6 +25,7 @@ Boston, MA 02111-1307, USA.  */
 //#include "14bit-registers.h"
 //#include "14bit-instructions.h"
 #include "../config.h"
+#include "packages.h"
 #include "16bit-processors.h"
 
 #include <string>
@@ -227,8 +228,10 @@ _16bit_processor::_16bit_processor(const char *_name, const char *desc)
   pc = new Program_Counter16(this);
   pc->set_trace_command(); //trace.allocateTraceType(new PCTraceType(this,1)));
 
-  m_porta = new PicPortRegister(this,"porta","",8,0x7f);
+  m_porta = new PicPortRegister(this,"porta","",8,0xff);
+  m_porta->setEnableMask(0x7f);
   m_trisa = new PicTrisRegister(this,"trisa","", m_porta, false);
+  m_trisa->setEnableMask(0x7f);
   m_lata  = new PicLatchRegister(this,"lata","", m_porta);
   m_lata->setEnableMask(0x7f);
 
@@ -566,12 +569,7 @@ void _16bit_processor::interrupt ()
 
   intcon.clear_gies();  // The appropriate gie bits get cleared (not just gieh)
 
-#if defined(CLOCK_EXPERIMENTS)
   pc->interrupt(intcon.get_interrupt_vector());
-#else
-  pc->interrupt(intcon.get_interrupt_vector());
-  //pc->jump(intcon.get_interrupt_vector());
-#endif
 
 }
 
@@ -618,6 +616,10 @@ unsigned int _16bit_processor::get_program_memory_at_address(unsigned int addres
   if (address >= CONFIG1L && address <= 0x30000D)
     return get_config_word(address);
 
+  uIndex = (address - 0x200000) >> 1;  // Look to see if it's an ID location
+  if( uIndex < IdentMemorySize() )
+    return idloc[uIndex];
+
   static const unsigned int DEVID1 = 0x3ffffe;
   static const unsigned int DEVID2 = 0x3fffff;
   if (address == DEVID1)
@@ -689,6 +691,54 @@ void _16bit_processor::create_config_memory()
   m_configMemory->addConfigWord(CONFIG2H-CONFIG1L,new Config2H(this, CONFIG2H));
 }
 
+
+void _16bit_processor::set_out_of_range_pm(unsigned int address, unsigned int value)
+{
+  // This method is only called by Processor::init_program_memory which writes words
+  if ( get_eeprom()
+    && (address>= 0xf00000)
+    && (address < 0xf00000 + get_eeprom()->get_rom_size()))
+    {
+        get_eeprom()->change_rom(1 + address - 0xf00000, value >> 8);
+        get_eeprom()->change_rom(address - 0xf00000, value & 0xff);
+    }
+  else if( (address>= 0x200000) && (address < 0x200008) ) {
+    idloc[(address - 0x200000) >> 1] = value;
+  }
+ 
+}
+void _16bit_processor::osc_mode(unsigned int value)
+{
+  IOPIN *m_pin;
+  unsigned int pin_Number =  get_osc_pin_Number(0);
+  
+  if (pin_Number < 253)
+  {
+	m_pin = package->get_pin(pin_Number);
+  }
+  if ( (pin_Number =  get_osc_pin_Number(1)) < 253 &&
+	(m_pin = package->get_pin(pin_Number)))
+  {
+	pll_factor = 0;
+        if (value < 5)
+	{
+	    set_clk_pin(pin_Number, m_osc_Monitor[1], "OSC2", true,
+		m_porta, m_trisa, m_lata);
+	}
+	else if(value == 6 )
+	{
+	    pll_factor = 2;
+	    set_clk_pin(pin_Number, m_osc_Monitor[1], "CLKO", false,
+		m_porta, m_trisa, m_lata);
+	}
+	else
+	{
+	    clr_clk_pin(pin_Number, m_osc_Monitor[1],
+		m_porta, m_trisa, m_lata);
+	}
+  }
+  
+}
 //-------------------------------------------------------------------
 //
 // Package stuff
