@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #include <stdio.h>
@@ -277,26 +277,22 @@ PLUSW::PLUSW(Processor *pCpu, const char *pName, const char *pDesc, Indirect_Add
 
 unsigned int PLUSW::get()
 {
-
   trace.raw(read_trace.get() | value.get());
 
   int destination = iam->plusw_fsr_value();
-  if(destination > 0)
-    return(cpu_pic->registers[destination]->get());
+  if(destination >= 0)
+    return (cpu_pic->registers[destination]->get());
   else
     return 0;
-
 }
 
 unsigned int PLUSW::get_value()
 {
-
   int destination = iam->plusw_fsr_value();
-  if(destination > 0)
-    return(cpu_pic->registers[destination]->get_value());
+  if(destination >= 0)
+    return (cpu_pic->registers[destination]->get_value());
   else
     return 0;
-
 }
 
 void PLUSW::put(unsigned int new_value)
@@ -305,21 +301,20 @@ void PLUSW::put(unsigned int new_value)
   //trace.register_write(address,new_value);
 
   int destination = iam->plusw_fsr_value();
-  if(destination > 0)
+  if(destination >= 0)
     cpu_pic->registers[destination]->put(new_value);
 }
 
 void PLUSW::put_value(unsigned int new_value)
 {
   int destination = iam->plusw_fsr_value();
-  if(destination > 0)
+  if(destination >= 0)
     cpu_pic->registers[destination]->put_value(new_value);
 
 
   update();
-  if(destination > 0)
+  if(destination >= 0)
     cpu_pic->registers[destination]->update();
-
 }
 
 //------------------------------------------------
@@ -581,7 +576,10 @@ void Program_Counter16::computed_goto(unsigned int new_address)
   // Use the new_address and the cached pclath
   // to generate the destination address:
 
-  value = ( (new_address | cpu_pic->get_pclath_branching_modpcl() )>>1) & memory_size_mask;
+
+  value = ( (new_address | cpu_pic->get_pclath_branching_modpcl() )>>1);
+  if (value >= memory_size)
+	value -= memory_size;
 
   // see Update pcl comment in Program_Counter::increment()
   cpu_pic->pcl->value.put((value<<1) & 0xff);
@@ -594,11 +592,7 @@ void Program_Counter16::computed_goto(unsigned int new_address)
   // the instruction (i.e. via the ::increment() method). The second cycle occurs
   // here:
 
-#ifdef CLOCK_EXPERIMENTS
   mCurrentPhase = mExecute1Cycle;
-#else
-  get_cycles().increment();
-#endif
 }
 
 //--------------------------------------------------
@@ -613,7 +607,9 @@ void Program_Counter16::put_value(unsigned int new_value)
 
   // RP - The new_value passed in is a byte address, but the Program_Counter16
   // class's internal value is a word address
-  value = (new_value/2) & memory_size_mask;
+  value = (new_value/2);
+  if (value >= memory_size)
+	value -= memory_size;
 
   cpu_pic->pcl->value.put(new_value & 0xfe);
 
@@ -1274,7 +1270,7 @@ void TBL_MODULE::read()
     ( (tabptrh.value.get() & 0xff) << 8 )  |
     ( (tabptrl.value.get() & 0xff) << 0 );
 
-  opcode = cpu_pic->pma->get_opcode(tabptr & 0xfffffe);
+  opcode = cpu_pic->pma->get_rom(tabptr & 0xfffffe);
 
   if(tabptr & 1)
     {
