@@ -2,22 +2,22 @@
    Copyright (C) 2006 T. Scott Dattalo
    Copyright (C) 2009 Roy R. Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include "ioports.h"
 #include "trace.h"
@@ -202,7 +202,7 @@ void ADCON0::put(unsigned int new_value)
   if(new_value & ADON) {
     // The A/D converter is being turned on (or maybe left on)
 
-    if((new_value & ~old_value) & GO) {
+    if((new_value & ~old_value) & GO_bit) {
       if (verbose)
 	printf("starting A2D conversion\n");
       Dprintf(("starting A2D conversion\n"));
@@ -232,7 +232,7 @@ void ADCON0::put_conversion(void)
   if (verbose)
 	printf ("result=0x%02x\n", converted);
 
-  Dprintf(("%d-bit result 0x%x\n", m_nBits, converted));
+  Dprintf(("%d-bit result 0x%x ADFM %d\n", m_nBits, converted, get_ADFM()));
 
   if(adresl) {   // non-null for more than 8 bit conversion
 
@@ -329,6 +329,7 @@ void ADCON0::set_interrupt(void)
 ADCON1_16F::ADCON1_16F(Processor *pCpu, const char *pName, const char *pDesc)
   : ADCON1(pCpu, pName, pDesc)
 {
+    valid_bits = 0x70;
 }
 void ADCON1_16F::put_value(unsigned int new_value)
 {
@@ -362,7 +363,7 @@ void ADCON1_16F::put_value(unsigned int new_value)
 
     }
     adcon0->set_Tad(Tad);
-    value.put(new_value);
+    value.put(new_value & valid_bits);
 }
 //------------------------------------------------------
 // ADCON1
@@ -370,7 +371,8 @@ void ADCON1_16F::put_value(unsigned int new_value)
 ADCON1::ADCON1(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
     m_AnalogPins(0),  m_voltageRef(0), m_nAnalogChannels(0),
-    mValidCfgBits(0), mCfgBitShift(0), mIoMask(0), m_ad_in_ctl(0)
+    mValidCfgBits(0), mCfgBitShift(0), mIoMask(0), 
+    valid_bits(0xff), m_ad_in_ctl(0)
 {
   for (int i=0; i<(int)cMaxConfigurations; i++) {
     setChannelConfiguration(i, 0);
@@ -381,14 +383,16 @@ ADCON1::ADCON1(Processor *pCpu, const char *pName, const char *pDesc)
 
 void ADCON1::put(unsigned int new_value)
 {
+  unsigned int masked_value = new_value & valid_bits;
   trace.raw(write_trace.get() | value.get());
-  put_value(new_value);
+  put_value(masked_value);
 }
 void ADCON1::put_value(unsigned int new_value)
 {
-    cfg_index = get_cfg(new_value);
+    unsigned int masked_value = new_value & valid_bits;
+    cfg_index = get_cfg(masked_value);
     setADCnames();
-    value.put(new_value);
+    value.put(masked_value);
 }
 
 // Obtain new mIoMask and set pin names as per function
@@ -397,7 +401,8 @@ void ADCON1::setADCnames()
     unsigned int new_mask = m_configuration_bits[cfg_index];
     unsigned int diff = mIoMask ^ new_mask;
 
-
+    Dprintf (( "ADCON1::setADCnames - cfg_index=%d new_mask %02X\n",
+               cfg_index, new_mask ));
 
     char newname[20];
 
@@ -457,6 +462,7 @@ unsigned int ADCON1::getVrefHiChannel(unsigned int cfg)
  */
 void ADCON1::setVrefLoConfiguration(unsigned int cfg, unsigned int channel)
 {
+  
   if (cfg < cMaxConfigurations) 
     Vreflo_position[cfg] = channel;
 }
@@ -556,6 +562,7 @@ void ADCON1::setVoltRef(unsigned int channel, float value)
 double ADCON1::getChannelVoltage(unsigned int channel)
 {
   double voltage=0.0;
+
   if(channel < m_nAnalogChannels) {
     if ( (1<<channel) & m_configuration_bits[cfg_index] ) {
       PinModule *pm = m_AnalogPins[channel];
@@ -627,9 +634,10 @@ void ADCON1::set_channel_in(unsigned int channel, bool on)
 
     m_AnalogPins[channel]->updatePinModule();
 }
+
 ANSEL::ANSEL(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    adcon1(0)
+    adcon1(0), anselh(0), valid_bits(0x7f)
 {
 }
 
@@ -642,7 +650,9 @@ void ANSEL::put(unsigned int new_value)
 {
   unsigned int cfgmax = adcon1->getNumberOfChannels();
   unsigned int i;
-  unsigned int mask = new_value;
+  unsigned int mask = (new_value & valid_bits);
+
+  if (anselh) mask |= anselh->value.get() << 8;
   trace.raw(write_trace.get() | value.get());
   /*
 	Generate ChannelConfiguration from ansel register
@@ -651,7 +661,39 @@ void ANSEL::put(unsigned int new_value)
   {
 	adcon1->setChannelConfiguration(i, mask);
   }
-  value.put(new_value);
+  value.put(new_value & valid_bits);
+  adcon1->setADCnames();
+}
+
+
+ANSEL_H::ANSEL_H(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    adcon1(0), ansel(0), valid_bits(0x3f)
+{
+}
+
+void ANSEL_H::setAdcon1(ADCON1 *new_adcon1)
+{
+  adcon1 = new_adcon1;
+}
+
+void ANSEL_H::put(unsigned int new_value)
+{
+  unsigned int cfgmax = adcon1->getNumberOfChannels();
+  unsigned int i;
+  unsigned int mask = ((new_value & valid_bits) << 8) ;
+  trace.raw(write_trace.get() | value.get());
+
+  if (ansel)
+	mask |= ansel->value.get();
+  /*
+	Generate ChannelConfiguration from ansel register
+  */
+  for(i=0; i < cfgmax; i++)
+  {
+	adcon1->setChannelConfiguration(i, mask);
+  }
+  value.put(new_value & valid_bits);
   adcon1->setADCnames();
 }
 
@@ -689,7 +731,7 @@ void ADCON0_10::put(unsigned int new_value)
 
   // If ADON is clear GO cannot be set
   if ((new_value & ADON) != ADON)
-	new_value &= ~GO;
+	new_value &= ~GO_bit;
 
 
   // SET: Reflect it first!
@@ -697,7 +739,7 @@ void ADCON0_10::put(unsigned int new_value)
   if(new_value & ADON) {
     // The A/D converter is being turned on (or maybe left on)
 
-    if((new_value & ~old_value) & GO) {
+    if((new_value & ~old_value) & GO_bit) {
       if (verbose)
 	printf("starting A2D conversion\n");
       // The 'GO' bit is being turned on, which is request to initiate
@@ -738,7 +780,7 @@ void ADCON0_12F::put(unsigned int new_value)
 
   // If ADON is clear GO cannot be set
   if ((new_value & ADON) != ADON)
-	new_value &= ~GO;
+	new_value &= ~GO_bit;
 
 
   // SET: Reflect it first!
@@ -746,7 +788,7 @@ void ADCON0_12F::put(unsigned int new_value)
   if(new_value & ADON) {
     // The A/D converter is being turned on (or maybe left on)
 
-    if((new_value & ~old_value) & GO) {
+    if((new_value & ~old_value) & GO_bit) {
       if (verbose)
 	printf("starting A2D conversion\n");
       // The 'GO' bit is being turned on, which is request to initiate
