@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #include <stdio.h>
@@ -182,6 +182,8 @@ PortRegister::PortRegister(Module *pCpu, const char *pName, const char *pDesc,
 
 void PortRegister::setEnableMask(unsigned int newEnableMask)
 {
+  Dprintf (( "PortRegister::setEnableMask for %s to %02X\n", 
+             name_str.c_str(), newEnableMask ));
   mOutputMask = newEnableMask;
   //unsigned int maskDiff = getEnableMask() ^ newEnableMask;
   unsigned int oldEnableMask = getEnableMask();
@@ -289,6 +291,8 @@ unsigned int PortRegister::get()
 }
 unsigned int PortRegister::get_value()
 {
+  Dprintf (( "PortRegister::get_value of %s mask=%02X, data=%02X\n",
+              name_str.c_str(), mOutputMask, rvDrivenValue.data ));
   return mOutputMask & rvDrivenValue.data;
 }
 void PortRegister::putDrive(unsigned int new_value)
@@ -452,6 +456,8 @@ PinModule::~PinModule()
   if (m_defaultPullupControl)
     m_defaultPullupControl->release();
 
+  if (m_pin)
+    m_pin->setMonitor(0);
 }
 
 void PinModule::setPin(IOPIN *new_pin)
