@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 //
@@ -101,6 +101,7 @@ void P16F62x::create_sfr_map()
   alias_file_registers(0x70,0x7f,0x80);
   alias_file_registers(0x70,0x7f,0x100);
   alias_file_registers(0x70,0x7f,0x180);
+  alias_file_registers(0x0,0x0,0x100);      // INDF exists in all four pages, 16x6x did the first two
   alias_file_registers(0x0,0x0,0x180);
   alias_file_registers(0x01,0x04,0x100);
   alias_file_registers(0x81,0x84,0x100);
@@ -120,9 +121,11 @@ void P16F62x::create_sfr_map()
   add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x9d);
 
   // PCLATH
+  alias_file_registers(0x0a,0x0a,0x100);
   alias_file_registers(0x0a,0x0a,0x180);
 
   add_sfr_register(&intcon_reg, 0x00b, RegisterValue(0,0));
+  alias_file_registers(0x0b,0x0b,0x100);
   alias_file_registers(0x0b,0x0b,0x180);
 
   usart.initialize(get_pir_set(),&(*m_portb)[2], &(*m_portb)[1],
