@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 //
@@ -25,7 +25,6 @@ Boston, MA 02111-1307, USA.  */
 //  This file supports:
 //    P16C61
 
-#define PORT_EXPERIMENT true
 
 
 #include <stdio.h>
@@ -73,7 +72,8 @@ P16C61::P16C61(const char *_name, const char *desc)
 }
 P16C61::~P16C61()
 {
-  cout << __FUNCTION__ << endl;
+  if (verbose)
+    cout << __FUNCTION__ << endl;
 }
 
 //------------------------------------------------------------------------
@@ -522,7 +522,8 @@ P16C63::P16C63(const char *_name, const char *desc)
 
 P16C63::~P16C63()
 {
-  cout << __FUNCTION__ << endl;
+  if (verbose)
+    cout << __FUNCTION__ << endl;
 
   delete_file_registers(0xc0, 0xff);
   delete_file_registers(0x19, 0x1a);  // usart tx and rx registers.
@@ -654,7 +655,8 @@ P16C64::P16C64(const char *_name, const char *desc)
 }
 P16C64::~P16C64()
 {
-  cout << __FUNCTION__ << endl;
+  if (verbose)
+    cout << __FUNCTION__ << endl;
 
   //delete_sfr_register(m_portc,0x07);
   //delete_sfr_register(m_trisc,0x87);
@@ -739,7 +741,8 @@ P16C65::P16C65(const char *_name, const char *desc)
 }
 P16C65::~P16C65()
 {
-  cout << __FUNCTION__ << endl;
+  if (verbose)
+    cout << __FUNCTION__ << endl;
 
   delete_file_registers(0xc0, 0xff);
 }
