@@ -1,22 +1,23 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
+   Copyright (C) 2010 Roy R Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #include <stdio.h>
@@ -596,15 +597,6 @@ void P18F242::create_sfr_map()
 }
 */
 
-void P18F242::set_out_of_range_pm(unsigned int address, unsigned int value)
-{
-
-  if( (address>= 0xf00000) && (address < 0xf00000 +
-    get_eeprom()->get_rom_size()))
-    {
-      get_eeprom()->change_rom(address - 0xf00000, value);
-    }
-}
 
 Processor * P18F242::construct(const char *name)
 {
@@ -710,15 +702,7 @@ void P18F442::create_sfr_map()
 
 }
 */
-void P18F442::set_out_of_range_pm(unsigned int address, unsigned int value)
-{
 
-  if( (address>= 0xf00000) && (address < 0xf00000 +
-    get_eeprom()->get_rom_size()))
-    {
-      get_eeprom()->change_rom(address - 0xf00000, value);
-    }
-}
 
 Processor * P18F442::construct(const char *name)
 {
@@ -851,11 +835,13 @@ Processor * P18F452::construct(const char *name)
 
 //------------------------------------------------------------------------
 //
-// P18F2455
+// P18F2455	- 18 pin
 // 
 
 P18F2455::P18F2455(const char *_name, const char *desc)
-  : P18F242(_name,desc)
+  : P18F242(_name,desc),
+    eccpas(this, "eccp1as", "ECCP Auto-Shutdown Control Register"),
+    pwm1con(this, "eccp1del", "Enhanced PWM Control Register")
 
 {
 
@@ -886,6 +872,15 @@ void P18F2455::create()
        );
 
   m_configMemory->addConfigWord(CONFIG3H-CONFIG1L,new Config3H_2x21(this, CONFIG3H, 0x83));
+  add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  ccp1con.setBitMask(0x3f);
+  pwm1con.set_mask(0x80);
+  eccpas.set_mask(0xfc);
+  eccpas.setIOpin(0, 0, &(*m_portb)[0]);
+  eccpas.link_registers(&pwm1con, &ccp1con);
+
+  ccp1con.setIOpin(&((*m_portc)[2]), 0, 0, 0);
 }
 
 Processor * P18F2455::construct(const char *name)
@@ -908,7 +903,9 @@ Processor * P18F2455::construct(const char *name)
 // 
 
 P18F4455::P18F4455(const char *_name, const char *desc)
-  : P18F442(_name,desc)
+  : P18F442(_name,desc),
+    eccpas(this, "eccp1as", "ECCP Auto-Shutdown Control Register"),
+    pwm1con(this, "eccp1del", "Enhanced PWM Control Register")
 
 {
 
@@ -939,6 +936,15 @@ void P18F4455::create()
        );
 
   m_configMemory->addConfigWord(CONFIG3H-CONFIG1L,new Config3H_2x21(this, CONFIG3H, 0x83));
+  add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  eccpas.setIOpin(0, 0, &(*m_portb)[0]);
+  eccpas.link_registers(&pwm1con, &ccp1con);
+//RRR  comparator.cmcon.set_eccpas(&eccpas);
+  ccp1con.setBitMask(0xff);
+  ccp1con.setCrosslinks(&ccpr1l, &pir1, &tmr2, &eccpas);
+  ccp1con.pwm1con = &pwm1con;
+  ccp1con.setIOpin(&((*m_portc)[2]), &((*m_portd)[5]), &((*m_portd)[6]), &((*m_portd)[7]));
 }
 
 Processor * P18F4455::construct(const char *name)
@@ -984,11 +990,29 @@ Processor * P18F1220::construct(const char *name)
 
 void P18F1220::create()
 {
+  EEPROM_PIR *e;
+
   if(verbose)
     cout << "P18F1220::create\n";
 
+  e = new EEPROM_PIR(this,&pir2);
+
+  // We might want to pass this value in for larger eeproms
+  e->initialize(256);
+  //e->set_pir_set(&pir_set_def);
+  e->set_intcon(&intcon);
+
+  // assign this eeprom to the processor
+  set_eeprom_pir(e);
+
   create_iopin_map();
 
+  //call his before _16bit_processor::create() because 1st one wins
+  //with regards to the rx/tx ports
+  usart.initialize(&pir_set_def,&(*m_portb)[1], &(*m_portb)[4],
+		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
+                   new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
+
   _16bit_processor::create();
   _16bit_v2_adc::create(7);
   adcon1->setIOPin(4, &(*m_portb)[0]);
@@ -1007,11 +1031,23 @@ void P18F1220::create()
   osccon.set_osctune(&osctune);
   osctune.set_osccon(&osccon);
 
+
   set_osc_pin_Number(0,16, &(*m_porta)[7]);
   set_osc_pin_Number(1,15, &(*m_porta)[6]);
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0xcf));
   m_configMemory->addConfigWord(CONFIG3H-CONFIG1L,new Config3H_1x20(this, CONFIG3H, 0x80));
 
+  add_sfr_register(&usart.baudcon,  0xfaa,RegisterValue(0,0),"baudcon");
+  usart.set_eusart(true);
+
+  add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  ccp1con.setBitMask(0xff);
+  ccp1con.setCrosslinks(&ccpr1l, &pir1, &tmr2, &eccpas);
+  eccpas.setIOpin(&(*m_portb)[1], &(*m_portb)[2], &(*m_portb)[0]);
+  eccpas.link_registers(&pwm1con, &ccp1con);
+  ccp1con.pwm1con = &pwm1con;
+  ccp1con.setIOpin(&((*m_portb)[3]), &((*m_portb)[2]), &((*m_portb)[6]), &((*m_portb)[7]));
 }
 //------------------------------------------------------------------------
 void P18F1220::create_iopin_map()
@@ -1047,7 +1083,10 @@ void P18F1220::create_iopin_map()
 
 P18F1220::P18F1220(const char *_name, const char *desc)
   : P18Fxx20(_name,desc),
-    osctune(this, "osctune", "OSC Tune")
+    osctune(this, "osctune", "OSC Tune"),
+    eccpas(this, "eccpas", "ECCP Auto-Shutdown Control Register"),
+    pwm1con(this, "pwm1con", "Enhanced PWM Control Register")
+
 {
 
   if(verbose)
@@ -1061,19 +1100,20 @@ void P18F1220::osc_mode(unsigned int value)
   IOPIN *m_pin;
   unsigned int pin_Number =  get_osc_pin_Number(0);
   
-  
   set_int_osc(false);
   if (pin_Number < 253)
   {
 	m_pin = package->get_pin(pin_Number);
-	if (value == 7 || value == 8)
+	if (value == 8 || value == 9)
 	{
-	    clr_clk_pin(pin_Number, get_osc_PinMonitor(0));
+	    clr_clk_pin(pin_Number, get_osc_PinMonitor(0), 
+		m_porta, m_trisa, m_lata);
 	    set_int_osc(true);
 	}
 	else
 	{
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(0), "OSC1", true);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(0), "OSC1", true,
+		 m_porta, m_trisa, m_lata);
 	}
   }
   if ( (pin_Number =  get_osc_pin_Number(1)) < 253 &&
@@ -1087,7 +1127,8 @@ void P18F1220::osc_mode(unsigned int value)
 	case 0:
 	case 1:
 	case 2:
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(1), "OSC2", true);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(1), "OSC2", true,
+		m_porta, m_trisa, m_lata);
 	    break;
 
 	case 4:
@@ -1097,17 +1138,21 @@ void P18F1220::osc_mode(unsigned int value)
 	case 14:
 	case 15:
 	    cout << "CLKO not simulated\n";
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(1) , "CLKO", false);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(1) , "CLKO", false,
+		m_porta, m_trisa, m_lata);
 	    break;
 
 	default:
-	    clr_clk_pin(pin_Number, get_osc_PinMonitor(1));
+	    clr_clk_pin(pin_Number, get_osc_PinMonitor(1),
+		m_porta, m_trisa, m_lata);
 	    break;
 	}
   }
   
 }
 
+
+
 //------------------------------------------------------------------------
 //
 // P18Fx320
@@ -1245,6 +1290,8 @@ void P18F2x21::create_symbols()
 
 P18F2x21::P18F2x21(const char *_name, const char *desc)
   : _16bit_v2_adc(_name,desc),
+    eccpas(this, "eccp1as", "ECCP Auto-Shutdown Control Register"),
+    pwm1con(this, "eccp1del", "Enhanced PWM Control Register"),
     osctune(this, "osctune", "OSC Tune"),
     comparator(this)
 {
@@ -1327,15 +1374,6 @@ void P18F2x21::create_sfr_map()
   usart.set_eusart(true);
 }
 
-void P18F2x21::set_out_of_range_pm(unsigned int address, unsigned int value)
-{
-
-  if( (address>= 0xf00000) && (address < 0xf00000 +
-    get_eeprom()->get_rom_size()))
-    {
-      get_eeprom()->change_rom(address - 0xf00000, value);
-    }
-}
 
 
 //------------------------------------------------------------------------
@@ -1374,6 +1412,17 @@ void P18F2321::create()
   set_osc_pin_Number(0, 9, &(*m_porta)[7]);
   set_osc_pin_Number(1,10, &(*m_porta)[6]);
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0x07));
+  add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  eccpas.set_mask(0xfc);
+  eccpas.setIOpin(0, 0, &(*m_portb)[0]);
+  eccpas.link_registers(&pwm1con, &ccp1con);
+  comparator.cmcon.set_eccpas(&eccpas);
+  ccp1con.setBitMask(0x3f);
+  ccp1con.setCrosslinks(&ccpr1l, &pir1, &tmr2, &eccpas);
+  ccp1con.pwm1con = &pwm1con;
+  ccp1con.setIOpin(&((*m_portc)[2]), 0, 0, 0);
+  pwm1con.set_mask(0x80);
 }
 
 Processor * P18F2321::construct(const char *name)
@@ -1404,12 +1453,14 @@ void P18F2321::osc_mode(unsigned int value)
 	m_pin = package->get_pin(pin_Number);
 	if (value == 8 || value == 9)	// internal RC clock
 	{
-	    clr_clk_pin(pin_Number, get_osc_PinMonitor(0));
+	    clr_clk_pin(pin_Number, get_osc_PinMonitor(0),
+		m_porta, m_trisa, m_lata);
 	    set_int_osc(true);
 	}
 	else
         {
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(0), "OSC1", true);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(0), "OSC1", true,
+		m_porta, m_trisa, m_lata);
 	    set_int_osc(false);
 	}
   }
@@ -1424,7 +1475,8 @@ void P18F2321::osc_mode(unsigned int value)
 	case 0:
 	case 1:
 	case 2:
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(1), "OSC2", true);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(1), "OSC2", true,
+		m_porta, m_trisa, m_lata);
 	    break;
 
 	case 3:
@@ -1437,11 +1489,13 @@ void P18F2321::osc_mode(unsigned int value)
 	case 14:
 	case 15:
 	    cout << "CLKO not simulated\n";
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(1) , "CLKO", false);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(1) , "CLKO", false,
+		m_porta, m_trisa, m_lata);
 	    break;
 
 	default:
-	    clr_clk_pin(pin_Number, get_osc_PinMonitor(1));
+	    clr_clk_pin(pin_Number, get_osc_PinMonitor(1),
+		m_porta, m_trisa, m_lata);
 	    break;
 	}
   }
@@ -1566,11 +1620,12 @@ P18F4x21::P18F4x21(const char *_name, const char *desc)
   if(verbose)
     cout << "18f4x21 constructor, type = " << isa() << '\n';
 
-  m_portd = new PicPSP_PortRegister(this,"portd","",8,0x08);
+  m_portd = new PicPSP_PortRegister(this,"portd","",8,0xFF);
   m_trisd = new PicTrisRegister(this,"trisd","", (PicPortRegister *)m_portd, true);
   m_latd  = new PicLatchRegister(this,"latd","",m_portd);
 
-  //m_porte = new PicPortRegister(this,"porte","",8,0x08);
+//  m_porte = new PicPortRegister(this,"porte","",8,0x07);
+  m_porte->setEnableMask(0x07);     // It's been created by the P18F2x21 constructor, but with the wrong enables
   m_trise = new PicPSP_TrisRegister(this,"trise","", m_porte, true);
   m_late  = new PicLatchRegister(this,"late","",m_porte);
 
@@ -1697,6 +1752,15 @@ void P18F4321::create()
   set_osc_pin_Number(0, 13, &(*m_porta)[7]);
   set_osc_pin_Number(1,14, &(*m_porta)[6]);
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0x07));
+  add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  eccpas.setIOpin(0, 0, &(*m_portb)[0]);
+  eccpas.link_registers(&pwm1con, &ccp1con);
+  comparator.cmcon.set_eccpas(&eccpas);
+  ccp1con.setBitMask(0xff);
+  ccp1con.setCrosslinks(&ccpr1l, &pir1, &tmr2, &eccpas);
+  ccp1con.pwm1con = &pwm1con;
+  ccp1con.setIOpin(&((*m_portc)[2]), &((*m_portd)[5]), &((*m_portd)[6]), &((*m_portd)[7]));
 
 }
 
@@ -1728,14 +1792,16 @@ void P18F4321::osc_mode(unsigned int value)
   if (pin_Number < 253)
   {
 	m_pin = package->get_pin(pin_Number);
-	if (value == 7 || value == 8)
+	if (value == 8 || value == 9)
 	{
 	    set_int_osc(true);
-	    clr_clk_pin(pin_Number, get_osc_PinMonitor(0));
+	    clr_clk_pin(pin_Number, get_osc_PinMonitor(0),
+		m_porta, m_trisa, m_lata);
 	}
 	else
 	{
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(0), "OSC1", true);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(0), "OSC1", true,
+		m_porta, m_trisa, m_lata);
 	}
   }
   if ( (pin_Number =  get_osc_pin_Number(1)) < 253 &&
@@ -1749,7 +1815,8 @@ void P18F4321::osc_mode(unsigned int value)
 	case 0:
 	case 1:
 	case 2:
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(1), "OSC2", true);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(1), "OSC2", true,
+		m_porta, m_trisa, m_lata);
 	    break;
 
 	case 3:
@@ -1762,11 +1829,13 @@ void P18F4321::osc_mode(unsigned int value)
 	case 14:
 	case 15:
 	    cout << "CLKO not simulated\n";
-	    set_clk_pin(pin_Number, get_osc_PinMonitor(1) , "CLKO", false);
+	    set_clk_pin(pin_Number, get_osc_PinMonitor(1) , "CLKO", false,
+		m_porta, m_trisa, m_lata);
 	    break;
 
 	default:
-	    clr_clk_pin(pin_Number, get_osc_PinMonitor(1));
+	    clr_clk_pin(pin_Number, get_osc_PinMonitor(1),
+		m_porta, m_trisa, m_lata);
 	    break;
 	}
   }
