@@ -2,22 +2,22 @@
    Copyright (C) 1998 Scott Dattalo
    Copyright (C) 2009 Roy R. Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #include <stdio.h>
@@ -31,8 +31,7 @@ Boston, MA 02111-1307, USA.  */
 #include "14bit-processors.h"  // %%% FIXME %%% remove the dependencies on this
 #include "pic-ioports.h"
 #include "interface.h"
-#include "p16x6x.h"
-//#include "p16f62x.h"
+#include "psp.h"
 
 #include "xref.h"
 //#define DEBUG
@@ -157,7 +156,7 @@ PicTrisRegister::PicTrisRegister(Processor *pCpu, const char *pName, const char
 void PicTrisRegister::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.data);
-  value.data = new_value;
+  value.data = new_value & m_EnableMask;
   if (m_port)
     m_port->updatePort();
 }
@@ -296,7 +295,10 @@ void PicPortBRegister::setbit(unsigned int bit_number, char new3State)
   bool bNewValue = new3State=='1' || new3State=='W';
   if (bit_number == 0 && (((rvDrivenValue.data&1)==1)!=m_bIntEdge) 
       && (bNewValue == m_bIntEdge))
+  {
+    cpu_pic->exit_sleep();
     m_pIntcon->set_intf(true);
+  }
 
 
   RegisterValue lastDrivenValue = rvDrivenValue;
@@ -462,6 +464,7 @@ void PicLatchRegister::put_value(unsigned int new_value)
 }
 unsigned int PicLatchRegister::get()
 {
+    value.data = m_port->getDriving();
   trace.raw(read_trace.get()  | value.data);
   trace.raw(read_trace.geti() | value.init);
 
