@@ -2,22 +2,22 @@
    Copyright (C) 1998 T. Scott Dattalo
    Copyright (C) 2009 Roy R. Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #ifndef __PIC_IOPORTS_H__
 #define __PIC_IOPORTS_H__
@@ -50,10 +50,7 @@ public:
   virtual unsigned int get();
   virtual char get3StateBit(unsigned int bitMask);
   void setEnableMask(unsigned int);
-  unsigned int getEnableMask()
-  {
-    return m_EnableMask;
-  }
+  unsigned int getEnableMask() { return m_EnableMask; }
   void reset(RESET_TYPE r);
 protected:
   PicPortRegister *m_port;
