@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <typeinfo>
 #include <stdio.h>
@@ -47,6 +47,7 @@ Boston, MA 02111-1307, USA.  */
 #include "p16x5x.h"
 #include "p16f62x.h"
 #include "p16f8x.h"
+#include "p16f88x.h"
 #include "p16x8x.h"
 #include "p16f87x.h"
 #include "p16x6x.h"
@@ -67,7 +68,6 @@ guint64 simulation_start_cycle;
 
 
 #include "clock_phase.h"
-#ifdef CLOCK_EXPERIMENTS
 
 class phaseCaptureInterrupt : public ProcessorPhase
 {
@@ -135,7 +135,6 @@ void phaseCaptureInterrupt::firstHalf()
   mCurrentPhase = this;
 }
 
-#endif
 
 //================================================================================
 // Global Declarations
@@ -152,125 +151,135 @@ void phaseCaptureInterrupt::firstHalf()
 //
 
 ProcessorConstructor pP10F200(P10F200::construct ,
-                              "__10F200", "pic10f200",  "p10f200", "10f200");
+                              "__10F200",   "pic10f200",    "p10f200",  "10f200");
 ProcessorConstructor pP10F202(P10F202::construct ,
-                              "__10F202", "pic10f202",  "p10f202", "10f202");
+                              "__10F202",   "pic10f202",    "p10f202",  "10f202");
 ProcessorConstructor pP10F204(P10F204::construct ,
-                              "__10F204", "pic10f204",  "p10f204", "10f204");
+                              "__10F204",   "pic10f204",    "p10f204",  "10f204");
 ProcessorConstructor pP10F220(P10F220::construct ,
-                              "__10F220", "pic10f220",  "p10f220", "10f220");
+                              "__10F220",   "pic10f220",    "p10f220",  "10f220");
 ProcessorConstructor pP10F222(P10F222::construct ,
-                              "__10F222", "pic10f222",  "p10f222", "10f222");
+                              "__10F222",   "pic10f222",    "p10f222",  "10f222");
 ProcessorConstructor pP12C508(P12C508::construct ,
-                              "__12C508", "pic12c508",  "p12c508", "12c508");
+                              "__12C508",   "pic12c508",    "p12c508",  "12c508");
 ProcessorConstructor pP12C509(P12C509::construct ,
-                              "__12C509", "pic12c509",  "p12c509", "12c509");
+                              "__12C509",   "pic12c509",    "p12c509",  "12c509");
 ProcessorConstructor pP12CE518(P12CE518::construct ,
-                              "__12ce518", "pic12ce518",  "p12ce518", "12ce518");
+                              "__12ce518",  "pic12ce518",   "p12ce518", "12ce518");
 ProcessorConstructor pP12CE519(P12CE519::construct ,
-                              "__12ce519", "pic12ce519",  "p12ce519", "12ce519");
+                              "__12ce519",  "pic12ce519",   "p12ce519", "12ce519");
 ProcessorConstructor pP12F508(P12F508::construct ,
-                              "__12F508", "pic12f508",  "p12f508", "12f508");
+                              "__12F508",   "pic12f508",    "p12f508",  "12f508");
 ProcessorConstructor pP12F509(P12F509::construct ,
-                              "__12F509", "pic12f509",  "p12f509", "12f509");
+                              "__12F509",   "pic12f509",    "p12f509",  "12f509");
 ProcessorConstructor pP12F510(P12F510::construct ,
-                              "__12F510", "pic12f510",  "p12f510", "12f510");
+                              "__12F510",   "pic12f510",    "p12f510",  "12f510");
 ProcessorConstructor pP12F629(P12F629::construct ,
-                              "__12F629", "pic12f629",  "p12f629", "12f629");
+                              "__12F629",   "pic12f629",    "p12f629",  "12f629");
 ProcessorConstructor pP12F675(P12F675::construct ,
-                              "__12F675", "pic12f675",  "p12f675", "12f675");
+                              "__12F675",   "pic12f675",    "p12f675",  "12f675");
 ProcessorConstructor pP12F683(P12F683::construct ,
-                              "__12F683", "pic12f683",  "p12f683", "12f683");
+                              "__12F683",   "pic12f683",    "p12f683",  "12f683");
 ProcessorConstructor pP16C54(P16C54::construct ,
-                             "__16C54",   "pic16c54",   "p16c54", "16c54");
+                             "__16C54",     "pic16c54",     "p16c54",   "16c54");
 ProcessorConstructor pP16C55(P16C55::construct ,
-                             "__16C55",   "pic16c55",   "p16c55", "16c55");
+                             "__16C55",     "pic16c55",     "p16c55",   "16c55");
 ProcessorConstructor pP16C56(P16C56::construct ,
-                             "__16C56",   "pic16c56",   "p16c56", "16c56");
+                             "__16C56",     "pic16c56",     "p16c56",   "16c56");
 ProcessorConstructor pP16C61(P16C61::construct ,
-                             "__16C61",   "pic16c61",   "p16c61", "16c61");
-ProcessorConstructor pP16C84(P16C84::construct ,
-                             "__16C84",  "pic16c84",   "p16c84", "16c84");
-ProcessorConstructor pP16CR83(P16CR83::construct ,
-                              "__16CR83", "pic16cr83",  "p16cr83", "16cr83");
-ProcessorConstructor pP16CR84(P16CR84::construct ,
-                              "__16CR84", "pic16cr84",  "p16cr84", "16cr84");
-ProcessorConstructor pP16C71(P16C71::construct ,
-                             "__16C71",   "pic16c71",   "p16c71", "16c71");
-ProcessorConstructor pP16C712(P16C712::construct ,
-                              "__16C712",  "pic16c712",  "p16c712", "16c712");
-ProcessorConstructor pP16C716(P16C716::construct ,
-                              "__16C716",  "pic16c716",  "p16c716", "16c716");
+                             "__16C61",     "pic16c61",     "p16c61",   "16c61");
 ProcessorConstructor pP16C62(P16C62::construct ,
-                             "__16C62",   "pic16c62",   "p16c62", "16c62");
+                             "__16C62",     "pic16c62",     "p16c62",   "16c62");
 ProcessorConstructor pP16C62A(P16C62::construct ,
-                              "__16C62A", "pic16c62a",  "p16c62a", "16c62a");
+                              "__16C62A",   "pic16c62a",    "p16c62a",  "16c62a");
 ProcessorConstructor pP16CR62(P16C62::construct ,
-                              "__16CR62", "pic16cr62",  "p16cr62", "16cr62");
+                              "__16CR62",   "pic16cr62",    "p16cr62",  "16cr62");
 ProcessorConstructor pP16C63(P16C63::construct ,
-                             "__16C63",   "pic16c63",   "p16c63", "16c63");
+                             "__16C63",     "pic16c63",     "p16c63",   "16c63");
 ProcessorConstructor pP16C64(P16C64::construct ,
-                             "__16C64",   "pic16c64",   "p16c64", "16c64");
-ProcessorConstructor pP16C65A(P16C65::construct ,
-                             "__16C65A", "pic16c65a",  "p16c65a", "16c65a");
+                             "__16C64",     "pic16c64",     "p16c64",   "16c64");
 ProcessorConstructor pP16C65(P16C65::construct ,
-                             "__16C65",   "pic16c65",   "p16c65", "16c65");
+                             "__16C65",     "pic16c65",     "p16c65",   "16c65");
+ProcessorConstructor pP16C65A(P16C65::construct ,
+                             "__16C65A",    "pic16c65a",    "p16c65a",  "16c65a");
+ProcessorConstructor pP16C71(P16C71::construct ,
+                             "__16C71",     "pic16c71",     "p16c71",   "16c71");
+ProcessorConstructor pP16C712(P16C712::construct ,
+                              "__16C712",   "pic16c712",    "p16c712",  "16c712");
+ProcessorConstructor pP16C716(P16C716::construct ,
+                              "__16C716",   "pic16c716",    "p16c716",  "16c716");
 ProcessorConstructor pP16C72(P16C72::construct ,
-                             "__16C72",   "pic16c72",   "p16c72", "16c72");
+                             "__16C72",     "pic16c72",     "p16c72",   "16c72");
 ProcessorConstructor pP16C73(P16C73::construct ,
-                             "__16C73",   "pic16c73",   "p16c73", "16c73");
+                             "__16C73",     "pic16c73",     "p16c73",   "16c73");
 ProcessorConstructor pP16C74(P16C74::construct ,
-                             "__16C74",   "pic16c74",   "p16c74", "16c74");
+                             "__16C74",     "pic16c74",     "p16c74",   "16c74");
+ProcessorConstructor pP16C84(P16C84::construct ,
+                             "__16C84",     "pic16c84",     "p16c84",   "16c84");
+ProcessorConstructor pP16CR83(P16CR83::construct ,
+                              "__16CR83",   "pic16cr83",    "p16cr83",  "16cr83");
+ProcessorConstructor pP16CR84(P16CR84::construct ,
+                              "__16CR84",   "pic16cr84",    "p16cr84",  "16cr84");
 ProcessorConstructor pP16F73(P16F73::construct ,
-                             "__16F73",   "pic16f73",   "p16f73", "16f73");
+                             "__16F73",     "pic16f73",     "p16f73",   "16f73");
 ProcessorConstructor pP16F74(P16F74::construct ,
-                             "__16F74",   "pic16f74",   "p16f74", "16f74");
+                             "__16F74",     "pic16f74",     "p16f74",   "16f74");
 ProcessorConstructor pP16F83(P16F83::construct ,
-                             "__16F83",   "pic16f83",   "p16f83", "16f83");
+                             "__16F83",     "pic16f83",     "p16f83",   "16f83");
 ProcessorConstructor pP16F84(P16F84::construct ,
-                             "__16F84",   "pic16f84",   "p16f84", "16f84");
-ProcessorConstructor pP16F627(P16F627::construct ,
-                              "__16F627", "pic16f627",  "p16f627", "16f627");
-ProcessorConstructor pP16F627A(P16F627::construct ,
-                              "__16F627A", "pic16f627a",  "p16f627a", "16f627a");
-ProcessorConstructor pP16F628(P16F628::construct ,
-                              "__16F628", "pic16f628",  "p16f628", "16f628");
-ProcessorConstructor pP16F628A(P16F628::construct ,
-                              "__16F628A", "pic16f628a",  "p16f628a", "16f628a");
-ProcessorConstructor pP16F630(P16F630::construct ,
-                              "__16F630", "pic16f630",  "p16f630", "16f630");
-ProcessorConstructor pP16F648(P16F648::construct ,
-                              "__16F648", "pic16f648",  "p16f648", "16f648");
-ProcessorConstructor pP16F648A(P16F648::construct ,
-                              "__16F648A", "pic16f648a",  "p16f648a", "16f648a");
+                             "__16F84",     "pic16f84",     "p16f84",   "16f84");
 ProcessorConstructor pP16F87(P16F87::construct ,
-                              "__16F87", "pic16f87",  "p16f87", "16f87");
-ProcessorConstructor pP16F676(P16F676::construct ,
-                              "__16F676", "pic16f676",  "p16f676", "16f676");
+                              "__16F87",    "pic16f87",     "p16f87",   "16f87");
 ProcessorConstructor pP16F88(P16F88::construct ,
-                              "__16F88", "pic16f88",  "p16f88", "16f88");
+                              "__16F88",    "pic16f88",     "p16f88",   "16f88");
+ProcessorConstructor pP16F882(P16F882::construct ,
+                     "__16F882",    "pic16f882",     "p16f882",   "16f882");
+ProcessorConstructor pP16F883(P16F883::construct ,
+                     "__16F883",    "pic16f883",     "p16f883",   "16f883");
+ProcessorConstructor pP16F884(P16F884::construct ,
+                     "__16F884",    "pic16f884",     "p16f884",   "16f884");
+ProcessorConstructor pP16F886(P16F886::construct ,
+                     "__16F886",    "pic16f886",     "p16f886",   "16f886");
+ProcessorConstructor pP16F887(P16F887::construct ,
+                     "__16F887",    "pic16f887",     "p16f887",   "16f887");
+ProcessorConstructor pP16F627(P16F627::construct ,
+                              "__16F627",   "pic16f627",    "p16f627",  "16f627");
+ProcessorConstructor pP16F627A(P16F627::construct ,
+                              "__16F627A",  "pic16f627a",   "p16f627a", "16f627a");
+ProcessorConstructor pP16F628(P16F628::construct ,
+                              "__16F628",   "pic16f628",    "p16f628",  "16f628");
+ProcessorConstructor pP16F628A(P16F628::construct ,
+                              "__16F628A",  "pic16f628a",   "p16f628a", "16f628a");
+ProcessorConstructor pP16F630(P16F630::construct ,
+                              "__16F630",   "pic16f630",    "p16f630",  "16f630");
+ProcessorConstructor pP16F648(P16F648::construct ,
+                              "__16F648",   "pic16f648",    "p16f648",  "16f648");
+ProcessorConstructor pP16F648A(P16F648::construct ,
+                              "__16F648A",  "pic16f648a",   "p16f648a", "16f648a");
+ProcessorConstructor pP16F676(P16F676::construct ,
+                              "__16F676",   "pic16f676",    "p16f676",  "16f676");
 ProcessorConstructor pP16F818(P16F818::construct ,
-                              "__16F818", "pic16f818",  "p16f818", "16f818");
+                              "__16F818",   "pic16f818",    "p16f818",  "16f818");
 ProcessorConstructor pP16F819(P16F819::construct ,
-                              "__16F819", "pic16f819",  "p16f819", "16f819");
+                              "__16F819",   "pic16f819",    "p16f819",  "16f819");
 ProcessorConstructor pP16F871(P16F871::construct ,
-                              "__16F871", "pic16f871",  "p16f871", "16f871");
+                              "__16F871",   "pic16f871",    "p16f871",  "16f871");
 ProcessorConstructor pP16F873(P16F873::construct ,
-                              "__16F873", "pic16f873",  "p16f873", "16f873");
+                              "__16F873",   "pic16f873",    "p16f873",  "16f873");
 ProcessorConstructor pP16F874(P16F874::construct ,
-                              "__16F874", "pic16f874",  "p16f874", "16f874");
+                              "__16F874",   "pic16f874",    "p16f874",  "16f874");
 ProcessorConstructor pP16F876(P16F876::construct ,
-                              "__16F876", "pic16f876",  "p16f876", "16f876");
+                              "__16F876",   "pic16f876",    "p16f876",  "16f876");
 ProcessorConstructor pP16F877(P16F877::construct ,
-                              "__16F877", "pic16f877",  "p16f877", "16f877");
+                              "__16F877",   "pic16f877",    "p16f877",  "16f877");
 ProcessorConstructor pP16F873A(P16F873A::construct ,
-                              "__16F873a", "pic16f873a", "p16f873a", "16f873a");
+                              "__16F873a",  "pic16f873a",   "p16f873a", "16f873a");
 ProcessorConstructor pP16F874A(P16F874A::construct ,
-                              "__16F874a", "pic16f874a", "p16f874a", "16f874a");
+                              "__16F874a",  "pic16f874a",   "p16f874a", "16f874a");
 ProcessorConstructor pP16F876A(P16F876A::construct ,
-                              "__16F876a", "pic16f876a", "p16f876a", "16f876a");
+                              "__16F876a",  "pic16f876a",   "p16f876a", "16f876a");
 ProcessorConstructor pP16F877A(P16F877A::construct ,
-                              "__16F877a", "pic16f877a", "p16f877a", "16f877a");
+                              "__16F877a",  "pic16f877a",   "p16f877a", "16f877a");
 #ifdef P17C7XX  // code no longer works
 ProcessorConstructor pP17C7xx(P17C7xx::construct ,
                               "__17C7xx", "pic17c7xx",  "p17c7xx", "17c7xx");
@@ -281,44 +290,44 @@ ProcessorConstructor pP17C752(P17C752::construct ,
 ProcessorConstructor pP17C756(P17C756::construct ,
                               "__17C756", "pic17c756",  "p17c756", "17c756");
 ProcessorConstructor pP17C756A(P17C756A::construct ,
-                               "__17C756A", "pic17c756a",  "p17c756a", "17c756a");
+                               "__17C756A", "pic17c756a",   "p17c756a", "17c756a");
 ProcessorConstructor pP17C762(P17C762::construct ,
                               "__17C762", "pic17c762",  "p17c762", "17c762");
 ProcessorConstructor pP17C766(P17C766::construct ,
                               "__17C766", "pic17c766",  "p17c766", "17c766");
 #endif // P17C7XX
 ProcessorConstructor pP18C242(P18C242::construct ,
-                              "__18C242", "pic18c242",  "p18c242", "18c242");
+                              "__18C242",   "pic18c242",    "p18c242",  "18c242");
 ProcessorConstructor pP18C252(P18C252::construct ,
-                              "__18C252", "pic18c252",  "p18c252", "18c252");
+                              "__18C252",   "pic18c252",    "p18c252",  "18c252");
 ProcessorConstructor pP18C442(P18C442::construct ,
-                              "__18C442", "pic18c442",  "p18c442", "18c442");
+                              "__18C442",   "pic18c442",    "p18c442",  "18c442");
 ProcessorConstructor pP18C452(P18C452::construct ,
-                              "__18C452", "pic18c452",  "p18c452", "18c452");
+                              "__18C452",   "pic18c452",    "p18c452",  "18c452");
 ProcessorConstructor pP18F242(P18F242::construct ,
-                              "__18F242", "pic18f242",  "p18f242", "18f242");
+                              "__18F242",   "pic18f242",    "p18f242",  "18f242");
 ProcessorConstructor pP18F248(P18F248::construct ,
-                              "__18F248", "pic18f248",  "p18f248", "18f248");
+                              "__18F248",   "pic18f248",    "p18f248",  "18f248");
 ProcessorConstructor pP18F252(P18F252::construct ,
-                              "__18F252", "pic18f252",  "p18f252", "18f252");
+                              "__18F252",   "pic18f252",    "p18f252",  "18f252");
 ProcessorConstructor pP18F442(P18F442::construct ,
-                              "__18F442", "pic18f442",  "p18f442", "18f442");
+                              "__18F442",   "pic18f442",    "p18f442",  "18f442");
 ProcessorConstructor pP18F448(P18F448::construct ,
-                              "__18F448", "pic18f448",  "p18f448", "18f448");
+                              "__18F448",   "pic18f448",    "p18f448",  "18f448");
 ProcessorConstructor pP18F452(P18F452::construct,
-                              "__18F452", "pic18f452",  "p18f452", "18f452");
+                              "__18F452",   "pic18f452",    "p18f452",  "18f452");
 ProcessorConstructor pP18F1220(P18F1220::construct,
-                              "__18F1220", "pic18f1220",  "p18f1220", "18f1220");
+                              "__18F1220",  "pic18f1220",   "p18f1220", "18f1220");
 ProcessorConstructor pP18F1320(P18F1320::construct,
-                              "__18F1320", "pic18f1320",  "p18f1320", "18f1320");
-ProcessorConstructor pP18F2455(P18F2455::construct,
-                              "__18F2455", "pic18f2455",  "p18f2455", "18f2455");
-ProcessorConstructor pP18F4455(P18F4455::construct,
-                              "__18F4455", "pic18f4455",  "p18f4455", "18f4455");
+                              "__18F1320",  "pic18f1320",   "p18f1320", "18f1320");
 ProcessorConstructor pP18F2321(P18F2321::construct,
-                              "__18F2321", "pic18f2321",  "p18f2321", "18f2321");
+                              "__18F2321",  "pic18f2321",   "p18f2321", "18f2321");
+ProcessorConstructor pP18F2455(P18F2455::construct,
+                              "__18F2455",  "pic18f2455",   "p18f2455", "18f2455");
 ProcessorConstructor pP18F4321(P18F4321::construct,
-                              "__18F4321", "pic18f4321",  "p18f4321", "18f4321");
+                              "__18F4321",  "pic18f4321",   "p18f4321", "18f4321");
+ProcessorConstructor pP18F4455(P18F4455::construct,
+                              "__18F4455",  "pic18f4455",   "p18f4455", "18f4455");
 
 
 //========================================================================
@@ -398,11 +407,7 @@ void pic_processor::BP_set_interrupt()
 {
 
   m_pInterruptTT->record();
-#ifdef CLOCK_EXPERIMENTS
   mCaptureInterrupt->firstHalf();
-#else
-  bp.set_interrupt();
-#endif
 }
 
 //-------------------------------------------------------------------
@@ -412,22 +417,6 @@ void pic_processor::BP_set_interrupt()
 
 void pic_processor::sleep ()
 {
-#if !defined(CLOCK_EXPERIMENTS)
-  simulation_mode = eSM_SLEEPING;
-
-  if(!bp.have_sleep())
-    return;
-
-  do
-    {
-      get_cycles().increment();   // burn cycles until something wakes us
-    } while(bp.have_sleep() && !bp.have_halt());
-
-  if(!bp.have_sleep())
-    pc->increment();
-
-  simulation_mode = eSM_RUNNING;
-#endif
 }
 //-------------------------------------------------------------------
 //
@@ -440,15 +429,11 @@ void pic_processor::enter_sleep()
   status->put_PD(0);
 
   wdt.update();
-#ifdef CLOCK_EXPERIMENTS
   pc->increment();
   mCurrentPhase->setNextPhase(mIdle);
   mCurrentPhase = mIdle;
   mCurrentPhase->setNextPhase(mIdle);
   m_ActivityState = ePASleeping;
-#else
-  bp.set_sleep();
-#endif
 
 }
 
@@ -459,12 +444,8 @@ void pic_processor::enter_sleep()
 void pic_processor::exit_sleep()
 {
 
-#if defined(CLOCK_EXPERIMENTS)
   m_ActivityState = ePAActive;
   mCurrentPhase->setNextPhase(mExecute1Cycle);
-#else
-  bp.clear_sleep();
-#endif
 }
 
 //-------------------------------------------------------------------
@@ -474,11 +455,7 @@ void pic_processor::exit_sleep()
 bool pic_processor::is_sleeping()
 {
 
-#if defined(CLOCK_EXPERIMENTS)
   return m_ActivityState == ePASleeping;
-#else
-  return false; // don't know what to do and is not being used - RRR
-#endif
 }
 
 //-------------------------------------------------------------------
@@ -519,7 +496,13 @@ public:
   guint64 cycle_start;
   guint64 future_cycle;
   int warntimer;
-  guint64 period;
+  guint64 period;            // callback period in us
+
+//#define REALTIME_DEBUG
+  guint64 diffmax;
+  guint64 diffsum;
+  int diffsumct;
+  struct timeval stat_start;
 
   RealTimeBreakPoint()
   {
@@ -527,6 +510,9 @@ public:
     warntimer = 1;
     period = 1;
     future_cycle = 0;
+    diffsum=0;
+    diffsumct=0;
+    diffmax=0;
   }
 
   void start(Processor *active_cpu)
@@ -534,6 +520,10 @@ public:
     if(!active_cpu)
       return;
 
+    diffsum=0;
+    diffsumct=0;
+    diffmax=0;
+
     // Grab the system time and record the simulated pic's time.
     // We'll then set a break point a short time in the future
     // and compare how the two track.
@@ -541,12 +531,13 @@ public:
     cpu = active_cpu;
 
     gettimeofday(&tv_start,0);
+    stat_start=tv_start;
 
     cycle_start=get_cycles().get();
 
     guint64 fc = cycle_start+100;
 
-    cout << "real time start : " << future_cycle << '\n';
+    //cout << "real time start : " << cycle_start << '\n';
 
     if(future_cycle)
       get_cycles().reassign_break(future_cycle, fc, this);
@@ -560,63 +551,95 @@ public:
   void stop()
   {
 
-    // Clear any pending break point.
-    cout << "real time stop : " << future_cycle << '\n';
+    //cout << "real time stop : " << future_cycle << '\n';
 
+#ifdef REALTIME_DEBUG
+    dump_stats();
+#endif
+
+    // Clear any pending break point.
     if(future_cycle) {
       cout << " real time clearing\n";
       get_cycles().clear_break(this);
       future_cycle = 0;
+      if(realtime_mode_with_gui)
+      {
+        update_gui();
+      }
     }
 
   }
 
+#ifdef REALTIME_DEBUG
+  void dump_stats(void)
+  {
+    struct timeval tv;
+    gettimeofday(&tv,0);
+    double simulation_time = (tv.tv_sec-stat_start.tv_sec)+(tv.tv_usec-stat_start.tv_usec)/1000000.0; // in seconds
+    if(diffsumct>0 && simulation_time>0)
+    {
+      cout << dec << "Average realtime error: " << diffsum/diffsumct << " microseconds. Max: "<<diffmax<<endl;
+      if(realtime_mode_with_gui)
+        cout << "Number of realtime callbacks (gui refreshes) per second:";
+      else
+        cout << "Number of realtime callbacks per second:";
+      cout << diffsumct/(double)simulation_time << endl;
+    }
+    stat_start=tv;
+    diffsum=0;
+    diffsumct=0;
+    diffmax=0;
+  }
+#endif
+
   void callback()
   {
-    gint64 system_time;
-    double diff;
+    guint64 system_time;	// wall clock time since datum in micro seconds
+    guint64 simulation_time;	// simulation time since datum in micro seconds
+    guint64 diff_us;
     struct timeval tv;
 
     // We just hit the break point. A few moments ago we
     // grabbed a snap shot of the system time and the simulated
     // pic's time. Now we're going to compare the two deltas and
-    // see how well they've tracked. If the host is running
-    // way faster than the PIC, we'll put the host to sleep
-    // briefly.
+    // see how well they've tracked. 
+    //
+    // If the host is running faster than the PIC, we'll put the 
+    // host to sleep briefly.
+    //
+    // If the host is running slower than the PIC, lengthen the
+    // time between GUI updates.
 
 
     gettimeofday(&tv,0);
 
-    system_time = (tv.tv_sec-tv_start.tv_sec)*1000000+(tv.tv_usec-tv_start.tv_usec); // in micro-seconds
+    system_time = (tv.tv_sec-tv_start.tv_sec)*1000000ULL+(tv.tv_usec-tv_start.tv_usec); // in micro-seconds
 
-    diff = system_time - ((get_cycles().get()-cycle_start)*4.0e6*cpu->get_OSCperiod());
+    simulation_time = ((get_cycles().get()-cycle_start)*4.0e6*cpu->get_OSCperiod());
 
-    guint64  idiff;
-    if( diff < 0 )
+
+    if( simulation_time > system_time)
     {
         // we are simulating too fast
 
-        idiff = (guint64)(-diff/4);
+	diff_us = simulation_time - system_time;
 
-        if(idiff>1000)
-            period -= idiff/500;
-        if(period<1)
-            period=1;
-
-        // Then sleep for a while
-        if(idiff)
-          usleep((unsigned int)idiff);
+	if(period > diff_us)
+          period -= diff_us;
+        else
+          period=1;
+        usleep((unsigned int)diff_us);
     }
     else
     {
-      idiff = (guint64)(diff/4);
+	diff_us = system_time - simulation_time;
 
-        if(idiff>1000)
-            period+=idiff/500;
-        if(period>10000)
-            period=10000;
+	period+=diff_us;
 
-        if(idiff>1000000)
+        if(period>1000000)
+            period=1000000; // limit to a one second callback period
+
+        if(diff_us>1000000)
         {
             // we are simulating too slow
             if(warntimer<10)
@@ -631,7 +654,7 @@ public:
             warntimer=0;
     }
 
-    guint64 delta_cycles= (guint64)(100*period*cpu->get_frequency()/4000000);
+    guint64 delta_cycles= (guint64)(period*cpu->get_frequency()/4000000);
     if(delta_cycles<1)
       delta_cycles=1;
 
@@ -641,6 +664,22 @@ public:
         update_gui();
     }
 
+#ifdef REALTIME_DEBUG
+    if(tv.tv_sec<stat_start.tv_sec+10) {
+      diffsum+=diff_us;
+      diffsumct++;
+    }
+    else
+    {
+      dump_stats();
+    }
+    if(diff_us>diffmax)
+      diffmax=diff_us;
+
+    static guint64 oldtime = 0;
+    //cout<<dec<<"dt="<<(system_time-oldtime)/1000 << "\tdiff_us="<<diff_us<<"\tdelta_cycles="<<delta_cycles<<"\tperiod="<<period<<endl;
+    oldtime = system_time;
+#endif
 
     guint64 fc = get_cycles().get() + delta_cycles;
 
@@ -673,7 +712,6 @@ void pic_processor::save_state()
 //
 // run  -- Begin simulating and don't stop until there is a break.
 //
-#if defined(CLOCK_EXPERIMENTS)
 
 void pic_processor::run (bool refresh)
 {
@@ -688,20 +726,22 @@ void pic_processor::run (bool refresh)
   // If the first instruction we're simulating is a break point,
   // then ignore it.
 
+  if(realtime_mode)
+    realtime_cbp.start(active_cpu);
+
   simulation_start_cycle = get_cycles().get();
   bp.clear_global();
 
-  // Take one step to get past any break point.
   mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
 
-  //  do {
-
-    mCurrentPhase = mCurrentPhase->advance();
 
     do
       mCurrentPhase = mCurrentPhase->advance();
     while(!bp.global_break);
 
+  if(realtime_mode)
+    realtime_cbp.stop();
+
     /* FIXME
     if(bp.have_pm_write())
       pm_write();
@@ -728,90 +768,6 @@ void pic_processor::run (bool refresh)
 
 }
 
-#else
-
-void pic_processor::run (bool refresh)
-{
-  if(get_use_icd())
-  {
-    cout  << "WARNING: gui_refresh is not being called "
-          <<  __FILE__<<':'<<__LINE__<<endl;
-
-      simulation_mode = eSM_RUNNING;
-      icd_run();
-      while(!icd_stopped())
-      {
-        //#ifdef HAVE_GUI
-        //        if(use_gui)
-        //            gui_refresh();
-        //#endif
-      }
-      simulation_mode=eSM_STOPPED;
-      disassemble((signed int)pc->get_value(), (signed int)pc->get_value());
-      gi.simulation_has_stopped();
-      return;
-  }
-
-
-  if(simulation_mode != eSM_STOPPED) {
-    if(verbose)
-      cout << "Ignoring run request because simulation is not stopped\n";
-    return;
-  }
-
-  simulation_mode = eSM_RUNNING;
-
-  if(realtime_mode)
-    realtime_cbp.start(active_cpu);
-
-  // If the first instruction we're simulating is a break point, then ignore it.
-
-  simulation_start_cycle = get_cycles().get();
-
-  do {
-
-    // Take one step to get past any break point.
-    step(1,false);
-
-    do {
-      program_memory[pc->value]->execute();
-    } while(!bp.global_break);
-
-    if(bp.have_interrupt())
-      interrupt();
-
-    if(bp.have_sleep())
-      sleep();
-
-    if(bp.have_pm_write())
-      pm_write();
-
-    if(bp.have_socket_break()) {
-      cout << " socket break point \n";
-      Interface *i = gi.get_socket_interface();
-      if (i)
-        i->Update(0);
-      bp.clear_socket_break();
-    }
-
-  } while(!bp.global_break);
-
-
-  if(realtime_mode)
-    realtime_cbp.stop();
-
-  bp.clear_global();
-  trace.cycle_counter(get_cycles().get());
-
-  simulation_mode = eSM_STOPPED;
-
-  if(refresh) {
-    trace.dump_last_instruction();
-    gi.simulation_has_stopped();
-  }
-
-}
-#endif
 
 //-------------------------------------------------------------------
 //
@@ -849,7 +805,6 @@ void pic_processor::step (unsigned int steps, bool refresh)
 
   simulation_mode = eSM_SINGLE_STEPPING;
 
-#ifdef CLOCK_EXPERIMENTS
 
   mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
 
@@ -866,33 +821,6 @@ void pic_processor::step (unsigned int steps, bool refresh)
   get_trace().cycle_counter(get_cycles().get());
   if(refresh)
     trace_dump(0,1);
-#else
-  do {
-
-
-    if(bp.have_sleep() || bp.have_pm_write()) {
-
-      // If we are sleeping or writing to the program memory (18cxxx only)
-      // then step one cycle - but don't execute any code
-
-      get_cycles().increment();
-      if(refresh)
-        trace_dump(0,1);
-
-    }
-    else if(bp.have_interrupt())
-      interrupt();
-    else {
-
-      step_one(refresh);
-      get_trace().cycle_counter(get_cycles().get());
-      if(refresh)
-        trace_dump(0,1);
-
-    }
-
-  }  while(!bp.have_halt() && --steps>0);
-#endif // CLOCK_EXPERIMENTS
 
   bp.clear_halt();
   simulation_mode = eSM_STOPPED;
@@ -905,11 +833,7 @@ void pic_processor::step (unsigned int steps, bool refresh)
 //-------------------------------------------------------------------
 void pic_processor::step_cycle()
 {
-#ifdef CLOCK_EXPERIMENTS
   mCurrentPhase = mCurrentPhase->advance();
-#else
-  step(1,false);
-#endif
 }
 
 //
@@ -985,6 +909,7 @@ void pic_processor::finish()
 
   run_to_address( stack->contents[(stack->pointer-1) & stack->stack_mask]);
 
+  get_interface().simulation_has_stopped();
 }
 
 
@@ -1029,35 +954,27 @@ void pic_processor::reset (RESET_TYPE r)
       if(config_modes) config_modes->print();
     }
     bHaltSimulation = false;
-#ifdef CLOCK_EXPERIMENTS
     mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
     m_ActivityState = ePAActive;
-#endif
     break;
 
   case MCLR_RESET:
-#ifdef CLOCK_EXPERIMENTS
     mCurrentPhase = mCurrentPhase ? mCurrentPhase : mIdle;
     mCurrentPhase->setNextPhase(mIdle);
     m_ActivityState = ePAIdle;
-#endif
     break;
 
   case IO_RESET:
-#ifdef CLOCK_EXPERIMENTS
     mCurrentPhase = mExecute1Cycle;
     mCurrentPhase->setNextPhase(mExecute1Cycle);
     m_ActivityState = ePAActive;
-#endif
     break;
 
   case WDT_RESET:
   case EXIT_RESET:
-#ifdef CLOCK_EXPERIMENTS
     mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
     mCurrentPhase->setNextPhase(mExecute1Cycle);
     m_ActivityState = ePAActive;
-#endif
     break;
 
   default:
@@ -1085,14 +1002,12 @@ pic_processor::pic_processor(const char *_name, const char *_desc)
     m_MCLR(0), m_MCLR_Save(0), m_MCLRMonitor(0)
 {
 
-#ifdef CLOCK_EXPERIMENTS
    mExecute1Cycle    = new phaseExecute1Cycle(this);
    mExecute2ndHalf   = new phaseExecute2ndHalf(this);
    mExecuteInterrupt = new phaseExecuteInterrupt(this);
    mCaptureInterrupt = new phaseCaptureInterrupt(this);
    mIdle             = new phaseIdle(this);
    mCurrentPhase   = mExecute1Cycle;
-#endif
 
   m_Capabilities = eSTACK | eWATCHDOGTIMER;
 
@@ -1127,16 +1042,22 @@ pic_processor::~pic_processor()
   delete m_PCHelper;
   delete stack;
 
-#ifdef CLOCK_EXPERIMENTS
   delete mExecute1Cycle;
   delete mExecute2ndHalf;
   delete mExecuteInterrupt;
   delete mCaptureInterrupt;
   delete mIdle;
-#endif
 
   delete config_modes;
   delete m_configMemory;
+
+  if (m_MCLR)
+    m_MCLR->setMonitor(0);
+  if (m_MCLR_Save)
+    m_MCLR_Save->setMonitor(0);
+  if (m_MCLRMonitor)
+    delete m_MCLRMonitor;
+
 }
 //-------------------------------------------------------------------
 //
@@ -1282,7 +1203,7 @@ void pic_processor::init_program_memory (unsigned int memory_size)
 
   // The memory_size_mask is used by the branching instructions
 
-  pc->memory_size_mask = memory_size - 1;
+  pc->memory_size = memory_size;
 
   Processor::init_program_memory(memory_size);
 }
@@ -1290,6 +1211,7 @@ void pic_processor::init_program_memory (unsigned int memory_size)
 void pic_processor::create_symbols ()
 {
 
+
   if(verbose)
     cout << __FUNCTION__ << " register memory size = " << register_memory_size() << '\n';
 
@@ -1817,9 +1739,12 @@ private:
 // removes its control from it's port register
 //
 void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number, 
-				     PinModule *PinMod, 
-				     const char * name,
-				     bool in)
+		PinModule *PinMod, 
+		const char * name,
+		bool in,
+                PicPortRegister *m_port,
+                PicTrisRegister *m_tris,
+                PicLatchRegister *m_lat)
 {
   IOPIN *m_pin = package->get_pin(pkg_Pin_Number);
   if (name)
@@ -1828,6 +1753,16 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
         m_pin->newGUIname(package->get_pin_name(pkg_Pin_Number).c_str());
   if (PinMod)
   {
+	if (m_port)
+	{
+	    unsigned int mask = m_port->getEnableMask();
+	    mask &= ~(1<< PinMod->getPinNumber());
+	    m_port->setEnableMask(mask);
+	    if (m_tris)
+		m_tris->setEnableMask(mask);
+	    if (m_lat)
+		m_lat->setEnableMask(mask);
+	}
     	PinMod->setSource(new IO_SignalControl('0'));
     	PinMod->setControl(new IO_SignalControl(in ? '1' : '0'));
  	PinMod->updatePinModule();
@@ -1835,17 +1770,31 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
 }
 // This function reverses the effects of the previous function
 void pic_processor::clr_clk_pin(unsigned int pkg_Pin_Number, 
-				     PinModule *PinMod )
+		PinModule *PinMod,
+		PicPortRegister *m_port,
+		PicTrisRegister *m_tris,
+		PicLatchRegister *m_lat)
 {
   IOPIN *m_pin = package->get_pin(pkg_Pin_Number);
   m_pin->newGUIname(package->get_pin_name(pkg_Pin_Number).c_str());
   if (PinMod)
   {
+	if (m_port)
+	{
+	    unsigned int mask = m_port->getEnableMask();
+	    mask |= (1<< PinMod->getPinNumber());
+	    m_port->setEnableMask(mask);
+	    if (m_tris)
+		m_tris->setEnableMask(mask);
+	    if (m_lat)
+		m_lat->setEnableMask(mask);
+	}
     	PinMod->setSource(0);
     	PinMod->setControl(0);
  	PinMod->updatePinModule();
   }
 }
+
 void pic_processor::osc_mode(unsigned int value)
 {
   IOPIN *m_pin;
