@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998-2000 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #ifndef __PIC_PROCESSORS_H__
 #define __PIC_PROCESSORS_H__
@@ -108,6 +108,11 @@ enum PROCESSOR_TYPE
   _P16F876A_,
   _P16F877_,
   _P16F877A_,
+  _P16F882_,
+  _P16F883_,
+  _P16F884_,
+  _P16F886_,
+  _P16F887_,
   _P17C7xx_,
   _P17C75x_,
   _P17C752_,
@@ -277,6 +282,8 @@ enum IOPIN_TYPES
  *
  * All pic processors are derived from this class.
  */
+class PicTrisRegister;
+class PicLatchRegister;
 
 class pic_processor : public Processor
 {
@@ -399,11 +406,20 @@ public:
   PinModule * get_osc_PinMonitor(unsigned int i)
 	{ return m_osc_Monitor[i]; }
 
+
   void set_clk_pin(unsigned int pkg_Pin_Number,
-                                     PinModule *PinMod,
-                                     const char * name,
-                                     bool in);
-  void clr_clk_pin(unsigned int pkg_Pin_Number, PinModule *PinMod );
+                       PinModule *PinMod,
+                       const char * name,
+                       bool in,
+		PicPortRegister *m_port = 0,
+		PicTrisRegister *m_tris = 0,
+		PicLatchRegister *m_lat = 0
+				  );
+  void clr_clk_pin(unsigned int pkg_Pin_Number, PinModule *PinMod,
+		PicPortRegister *m_port = 0,
+		PicTrisRegister *m_tris = 0,
+		PicLatchRegister *m_lat = 0
+		 );
 
   virtual void set_int_osc(bool val){ internal_osc = val;}
   virtual bool get_int_osc(){ return internal_osc; }
