@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998-2003 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 /*
   stuff that needs to be fixed:
@@ -616,6 +616,16 @@ void Processor::build_program_memory(unsigned int *memory,
 }
 
 //-------------------------------------------------------------------
+/** @brief Write a word of data into memory outside flash
+ *
+ *  This method is called when loading data from the COD or HEX file
+ *  and the address is not in the program ROM or normal config space.
+ *  In this base class, there is no such memory. Real processors,
+ *  particularly those with EEPROM, will need to override this method.
+ *
+ *  @param  address Memory address to set. Byte address on 18F
+ *  @param  value   Word data to write in.
+ */
 void Processor::set_out_of_range_pm(unsigned int address, unsigned int value)
 {
 
@@ -1024,27 +1034,6 @@ int ProgramMemoryAccess::find_closest_address_to_line(int file_id, int src_line)
 
   return closest_address;
 
-#if 0
-  int distance = cpu->program_memory_size();
-
-  for(int i = cpu->program_memory_size()-1; i>=0; i--) {
-
-    // Find the closet instruction to the src file line number
-    if( (cpu->program_memory[i]->isa() != instruction::INVALID_INSTRUCTION) &&
-        (cpu->program_memory[i]->get_file_id()==file_id)                    &&
-        (abs(cpu->program_memory[i]->get_src_line() - src_line) < distance))
-
-      {
-        distance = abs(cpu->program_memory[i]->get_src_line() - src_line);
-        closest_address = i;
-
-        if(distance == 0)
-          return cpu->map_pm_index2address(closest_address);
-      }
-  }
-
-  return cpu->map_pm_index2address(closest_address);
-#endif
 
 }
 //-------------------------------------------------------------------
@@ -1068,7 +1057,7 @@ void ProgramMemoryAccess::set_hll_mode(unsigned int new_hll_mode)
   }
 }
 //--------------------------------------------------------------------------
-unsigned int ProgramMemoryAccess::get_src_line(unsigned int address)
+int ProgramMemoryAccess::get_src_line(unsigned int address)
 {
   unsigned int line=0;
 
@@ -1090,7 +1079,7 @@ unsigned int ProgramMemoryAccess::get_src_line(unsigned int address)
 }
 
 //--------------------------------------------------------------------------
-unsigned int ProgramMemoryAccess::get_file_id(unsigned int address)
+int ProgramMemoryAccess::get_file_id(unsigned int address)
 {
 
   if(!cpu)
@@ -1349,90 +1338,6 @@ RegisterValue Processor::getReadTT(unsigned int j)
   return RegisterValue(tt, tt + (1<<24) );
 }
 
-//-------------------------------------------------------------------
-//
-// run  -- Begin simulating and don't stop until there is a break.
-// FIXME - shouldn't this be a pure virtual function?
-#if 0
-void Processor::run (bool refresh)
-{
-
-  cout << __FUNCTION__ << endl;
-}
-#endif
-//-------------------------------------------------------------------
-//
-// step - Simulate one (or more) instructions. If a breakpoint is set
-// at the current PC-> 'step' will go right through it. (That's supposed
-// to be a feature.)
-//
-#if 0
-extern phaseExecute1Cycle *mExecute1Cycle;
-extern phaseExecute2ndHalf *mExecute2ndHalf;
-void Processor::step (unsigned int steps, bool refresh)
-{
-  if(!steps)
-    return;
-
-  if(simulation_mode != eSM_STOPPED) {
-    if(verbose)
-      cout << "Ignoring step request because simulation is not stopped\n";
-    return;
-  }
-
-  simulation_mode = eSM_SINGLE_STEPPING;
-#ifdef CLOCK_EXPERIMENTS
-
-  mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
-
-  do
-    mCurrentPhase = mCurrentPhase->advance();
-  while(!bp.have_halt() && --steps>0);
-
-  // complete the step if this is a multi-cycle instruction.
-
-  if (mCurrentPhase == mExecute2ndHalf)
-    while (mCurrentPhase != mExecute1Cycle)
-      mCurrentPhase = mCurrentPhase->advance();
-
-  get_trace().cycle_counter(get_cycles().get());
-  if(refresh)
-    trace_dump(0,1);
-#else
-  do {
-
-
-    if(bp.have_sleep() || bp.have_pm_write()) {
-
-      // If we are sleeping or writing to the program memory (18cxxx only)
-      // then step one cycle - but don't execute any code
-
-      get_cycles().increment();
-      if(refresh)
-        trace_dump(0,1);
-
-    }
-    else if(bp.have_interrupt())
-      interrupt();
-    else {
-
-      step_one(refresh);
-      get_trace().cycle_counter(get_cycles().get());
-      if(refresh)
-        trace_dump(0,1);
-
-    }
-
-  }  while(!bp.have_halt() && --steps>0);
-#endif // CLOCK_EXPERIMENTS
-
-  bp.clear_halt();
-  simulation_mode = eSM_STOPPED;
-
-  if(refresh)
-    get_interface().simulation_has_stopped();
-}
-#endif
 
 //-------------------------------------------------------------------
 //
@@ -2008,23 +1913,29 @@ void ProgramMemoryAccess::step(unsigned int steps,bool refresh)
     break;
 
   case HLL_MODE:
-    {
-      unsigned int initial_line = cpu->pma->get_src_line(cpu->pc->get_value());
-      unsigned int initial_pc = cpu->pc->get_value();
+    unsigned int initial_pc = cpu->pc->get_value();
+    int initial_line = cpu->pma->get_src_line(initial_pc);
+    int initial_file = cpu->pma->get_file_id(initial_pc);
 
-      while(1) {
-        cpu->step(1,false);
+    while(1) {
+      cpu->step(1,false);
 
-        if(( cpu->pc->get_value() == initial_pc) ||
-           (get_src_line(cpu->pc->get_value()) != initial_line)) {
-          if(refresh)
-            get_interface().simulation_has_stopped();
-          break;
-        }
+      unsigned int current_pc = cpu->pc->get_value();
+      int current_line = cpu->pma->get_src_line(current_pc);
+      int current_file = cpu->pma->get_file_id(current_pc);
+
+      if(current_line<0 || current_file<0)
+        continue;
+
+      if(current_pc == initial_pc ||
+         current_line != initial_line ||
+         current_file != initial_file) {
+        if(refresh)
+          get_interface().simulation_has_stopped();
+        break;
       }
-
-      break;
     }
+    break;
   }
 }
 
@@ -2042,44 +1953,39 @@ void ProgramMemoryAccess::step_over(bool refresh)
     break;
 
   case HLL_MODE:
-    {
-
-#if 0
-      //
-      // High level language step-overs are only supported for
-      // pic processors
-      //
-
-      pic_processor *pic = dynamic_cast<pic_processor *>cpu;
-
-      if(!pic) {
-        cout << "step-over is not supported for non-PIC processors\n";
-        return;
-      }
-      unsigned int initial_line = cpu->pma.get_src_line(cpu->pc->get_value());
-      unsigned int initial_pc = cpu->pc->get_value();
-      unsigned int initial_stack_depth = pic->stack->pointer&pic->stack->stack_mask;
-
-      while(1)
-        {
-          step(1);
-
-          if(initial_stack_depth < pic->stack->pointer&pic->stack->stack_mask)
-            gpsim_finish(processor_id);
-
-          if(pic->pc->get_value()==initial_pc)
-            break;
-
-          if(get_src_line(initial_pc) != initial_line)
-            {
-              if(gpsim_get_file_id(processor_id, pic->pc->get_value()))
-                break;
-            }
-        }
-      break;
-#endif
-      cout << "HLL Step is not supported\n";
+    pic_processor *pic = dynamic_cast<pic_processor *>(cpu);
+    if(!pic) {
+      cout << "step-over is not supported for non-PIC processors\n";
+      return;
     }
+
+    unsigned int initial_pc = cpu->pc->get_value();
+    int initial_line = cpu->pma->get_src_line(initial_pc);
+    int initial_file = cpu->pma->get_file_id(initial_pc);
+    unsigned int initial_stack_depth = pic->stack->pointer&pic->stack->stack_mask;
+
+    while(1) {
+      cpu->step(1,false);
+
+      if(initial_stack_depth < (pic->stack->pointer&pic->stack->stack_mask))
+        cpu->finish();
+
+      unsigned int current_pc = cpu->pc->get_value();
+      int current_line = cpu->pma->get_src_line(current_pc);
+      int current_file = cpu->pma->get_file_id(current_pc);
+
+      if(current_line<0 || current_file<0)
+        continue;
+
+      if(current_pc == initial_pc ||
+         current_line != initial_line ||
+         current_file != initial_file) {
+        if(refresh)
+          get_interface().simulation_has_stopped();
+        break;
+      }
+    }
+    break;
   }
 
 }
@@ -2389,26 +2295,22 @@ FileContext::FileContext(string &new_name)
 {
   name_str = new_name;
   fptr = NULL;
-  line_seek = 0;
   m_uiMaxLine = 0;
-  pm_address = 0;
   m_bIsList = false;
+  m_bIsHLL = false;
 }
 
 FileContext::FileContext(const char *new_name)
 {
   name_str = string(new_name);
   fptr = NULL;
-  line_seek = 0;
   m_uiMaxLine = 0;
-  pm_address = 0;
   m_bIsList = false;
+  m_bIsHLL = false;
 }
 
 FileContext::~FileContext(void)
 {
-  delete line_seek;
-  delete pm_address;
 }
 
 //----------------------------------------
@@ -2441,20 +2343,16 @@ void FileContext::ReadSource(void)
     return;
   }
 
-  delete line_seek;
-  delete pm_address;
-
-  line_seek = new vector<int>(max_line()+1);
-  pm_address = new vector<int>(max_line()+1);
+  line_seek.resize(max_line()+1);
+  pm_address.resize(max_line()+1);
 
   std::rewind(fptr);
 
   char buf[256],*s;
-  (*line_seek)[0] = 0;
+  line_seek[0] = 0;
   for(unsigned int j=1; j<=max_line(); j++) {
-
-    (*pm_address)[j] = -1;
-    (*line_seek)[j] = ftell(fptr);
+    pm_address[j] = -1;
+    line_seek[j] = ftell(fptr);
     s = fgets(buf,256,fptr);
 
     if(s != buf)
@@ -2474,7 +2372,7 @@ char *FileContext::ReadLine(unsigned int line_number, char *buf, unsigned int nB
     return 0;
 
   fseek(fptr,
-        (*line_seek)[line_number],
+        line_seek[line_number],
         SEEK_SET);
   return fgets(buf, nBytes, fptr);
 
@@ -2531,15 +2429,15 @@ void FileContext::close()
 //----------------------------------------
 int FileContext::get_address(unsigned int line_number)
 {
-  if(line_number <= max_line()  && pm_address)
-    return (*pm_address)[line_number];
+  if(line_number <= max_line() && pm_address.size() > line_number)
+    return pm_address[line_number];
   return -1;
 }
 //----------------------------------------
 void FileContext::put_address(unsigned int line_number, unsigned int address)
 {
-  if(line_number <= max_line()  && pm_address && (*pm_address)[line_number]<0)
-    (*pm_address)[line_number] = address;
+  if(line_number <= max_line()  && pm_address.size() > line_number && pm_address[line_number]<0)
+    pm_address[line_number] = address;
 }
 
 //------------------------------------------------------------------------
@@ -2585,11 +2483,20 @@ int FileContextList::Find(string &fname)
 
 extern bool bHasAbsolutePath(string &fname);
 
-int FileContextList::Add(string &new_name)
+int FileContextList::Add(string &new_name, bool hll)
 {
 
   string sFull = bHasAbsolutePath(new_name) ? new_name : (sSourcePath + new_name);
+
+  // Check that the file is available.
+  // FIXME stat() would be better, but would need some stat_path() I guess.
+  FILE *f=fopen_path(sFull.c_str(), "r");
+  if(f==NULL)
+    return -1;
+  fclose(f);
+
   push_back(FileContext(sFull));
+  back().setHLLId(hll);
   lastFile++;
   if(CSimulationContext::GetContext()->IsSourceEnabled()) {
     back().open("r");
@@ -2601,10 +2508,10 @@ int FileContextList::Add(string &new_name)
   return lastFile-1;
 }
 
-int FileContextList::Add(const char *new_name)
+int FileContextList::Add(const char *new_name, bool hll)
 {
   string sNewName(new_name);
-  return Add (sNewName);
+  return Add (sNewName, hll);
 }
 
 FileContext *FileContextList::operator [] (int file_id)
