@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998-2003 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #ifndef __PROCESSOR_H__
 #define __PROCESSOR_H__
@@ -138,10 +138,10 @@ class ProgramMemoryAccess :  public MemoryAccess
   // Given an address to an instruction, find the source line that 
   // created it:
 
-  unsigned int get_src_line(unsigned int address);
+  int get_src_line(unsigned int address);
 
   // Return the file ID of the source program responsible for the opcode at address.
-  unsigned int get_file_id(unsigned int address);
+  int get_file_id(unsigned int address);
 
   // A couple of functions for manipulating  breakpoints
   virtual unsigned int  set_break_at_address(unsigned int address);
@@ -242,18 +242,17 @@ class FileContext
 private:
   string name_str;           // File name
   FILE   *fptr;              // File ptr when the file is opened
-  vector<int> *line_seek;    // A vector of file offsets to the start of lines
-  vector<int> *pm_address;   // A vector of program memory addresses for lines 
+  vector<int> line_seek;     // A vector of file offsets to the start of lines
+  vector<int> pm_address;    // A vector of program memory addresses for lines 
   unsigned int m_uiMaxLine;  // number of lines in the file
 
   friend class FileContextList;
 protected:
   bool m_bIsList;          // True if this is a list file.
+  bool m_bIsHLL;           // True if this is a HLL file.
 
-  void setListId(bool b)
-  { 
-    m_bIsList = b; 
-  }
+  void setListId(bool b) { m_bIsList = b; }
+  void setHLLId(bool b) { m_bIsHLL = b; }
 public:
   // cache -- deprecated - this was used with the old gui source browser
   typedef vector<gpsimObject*> Cache;
@@ -271,6 +270,7 @@ public:
   void close();
   bool IsOpen() { return fptr != NULL; }
   bool IsList() { return m_bIsList; }
+  bool IsHLL() { return m_bIsHLL; }
 
   /// get_address - given a line number, return the program memory address
   int get_address(unsigned int line);
@@ -300,8 +300,8 @@ public:
   FileContextList();
   ~FileContextList();
 
-  int Add(string& new_name);
-  int Add(const char *new_name);
+  int Add(string& new_name, bool hll=false);
+  int Add(const char *new_name, bool hll=false);
 
   int Find(string &fname);
 
