@@ -2,22 +2,22 @@
    Copyright (C) 1998,1999,2000,2001,2002 Scott Dattalo
 	         2006 Roy R Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <stdio.h>
 #include <iostream>
@@ -31,6 +31,13 @@ Boston, MA 02111-1307, USA.  */
 
 #include "xref.h"
 
+//#define DEBUG
+#if defined(DEBUG)
+#define Dprintf(arg) {printf("%s:%d-%s() ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
+
 //#warning only supports SPI mode.
 //-----------------------------------------------------------
 // SSPSTAT - Synchronous Serial Port Status register.
@@ -328,6 +335,7 @@ void SPI::clock( bool ClockState )
 {
   // A clock has happened. Either we sent one or we recieved one.
   bool onbeat;
+  bool allDone = false;
   if( !m_sspstat || ! m_sspcon)
     return;
   unsigned int sspstat_val = m_sspstat->value.get();
@@ -367,6 +375,8 @@ void SPI::clock( bool ClockState )
     }
     else 
     {
+        // RP: This is only relevant in slave mode? I think clock is never called
+        // while idle in master mode.
       if (verbose)
       	cout << "SPI clock called start_transfer\n";
       start_transfer();
@@ -411,15 +421,18 @@ void SPI::clock( bool ClockState )
       if( (sspstat_val & _SSPSTAT::SMP) && !(sspstat_val & _SSPSTAT::CKE) ) 
       {
 	m_state = eWAITING_FOR_LAST_SMP;
-	set_halfclock_break();
       } 
       else 
       {
 	stop_transfer();
+        allDone = true;
       }
     }
   }
+  if ( !allDone && m_sspcon->isSPIMaster() )
+    set_halfclock_break();
 }
+
 void SPI::set_halfclock_break()
 {
   int clock_in_cycles = 1;
@@ -460,7 +473,6 @@ void SPI::callback()
   case eACTIVE:
     m_sspmod->Sck_toggle();
     clock( m_sspmod->get_SCL_State() );
-    set_halfclock_break();
     break;
   case eWAITING_FOR_LAST_SMP:
     if( m_sspstat && m_sspstat->value.get() & _SSPSTAT::SMP ) {
@@ -543,10 +555,13 @@ void SPI::start_transfer()
   case _SSPCON::SSPM_SPImaster4:
   case _SSPCON::SSPM_SPImaster16:
   case _SSPCON::SSPM_SPImaster64:
+    // In master mode, the SDO line is always set at the start of the transfer
+    m_sspmod->putStateSDO((m_SSPsr &(1<<7)) ? '1' : '0');
     // Setup callbacks for clocks
     set_halfclock_break();
     break;
   case _SSPCON::SSPM_SPImasterTMR2: 
+      m_sspmod->putStateSDO((m_SSPsr &(1<<7)) ? '1' : '0');
 	break;
   case _SSPCON::SSPM_SPIslaveSS:
     // The SS pin was pulled low
@@ -1582,12 +1597,12 @@ void SSP_MODULE::newSSPBUF(unsigned int value)
 {
   if (!m_spi)
   {
-	cout << "Warning bug, SPI initialization error " << __FILE__ << ":" << __LINE__<<endl;
+	cout << "Warning bug, SPI initialization error " << __FILE__ << ":" << dec << __LINE__<<endl;
 	return;
   }
   if (!m_i2c)
   {
-	cout << "Warning bug, I2C initialization error " << __FILE__ << ":" << __LINE__<<endl;
+	cout << "Warning bug, I2C initialization error " << __FILE__ << ":" << dec << __LINE__<<endl;
 	return;
   }
   if(sspcon.isSPIActive(sspcon.value.get()))
