@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998,1999 Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <stdio.h>
 #include <iostream>
@@ -63,6 +63,25 @@ private:
   _TXSTA *m_txsta;
 };
 
+class TXSignalControl : public SignalControl
+{
+public:
+  TXSignalControl()
+  {
+  }
+  ~TXSignalControl()
+  {
+  }
+  char getState()
+  {
+    return '0';
+  }
+  void release()
+  {
+    delete this;
+  }
+};
+
 //--------------------------------------------------
 //
 //--------------------------------------------------
@@ -108,7 +127,10 @@ _TXSTA::_TXSTA(Processor *pCpu, const char *pName, const char *pDesc, USART_MODU
   : sfr_register(pCpu, pName, pDesc), txreg(0), spbrg(0),
     mUSART(pUSART),
     m_PinModule(0),
-    m_source(0), m_cTxState('?'), bInvertPin(0)
+    m_source(0), 
+    m_control(0),
+    m_cTxState('?'),
+    bInvertPin(0)
 {
   assert(mUSART);
 }
@@ -138,7 +160,7 @@ _BAUDCON::_BAUDCON(Processor *pCpu, const char *pName, const char *pDesc)
 
 _SPBRG::_SPBRG(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    txsta(0), rcsta(0), brgh(0), baudcon(0)
+    txsta(0), rcsta(0), brgh(0), baudcon(0), skip(0)
 {
 }
 
@@ -148,7 +170,6 @@ _SPBRGH::_SPBRGH(Processor *pCpu, const char *pName, const char *pDesc)
 {
 }
 
-
 //-----------------------------------------------------------
 // TXREG - USART Transmit Register
 
@@ -197,6 +218,7 @@ void _TXSTA::setIOpin(PinModule *newPinModule)
 {
   if (!m_source) {
     m_source = new TXSignalSource(this);
+    m_control = new TXSignalControl();
     m_PinModule = newPinModule;
   }
 
@@ -257,14 +279,19 @@ void _TXSTA::put(unsigned int new_value)
 
     if(value.get() & TXEN) {
       Dprintf(("TXSTA - enabling transmitter\n"));
-      if (m_PinModule)
+      if (m_PinModule) {
         m_PinModule->setSource(m_source);
+// all set diection ? RRR        if(mUSART->IsEUSART()) // EUSART can configure input as output for transmit
+          m_PinModule->setControl(m_control);
+      }
       mUSART->emptyTX();
     } else {
       stop_transmitting();
-      if (m_PinModule)
+      if (m_PinModule) {
         m_PinModule->setSource(0);
-
+        if(mUSART->IsEUSART())
+          m_PinModule->setControl(0);
+      }
     }
   }
 }
@@ -370,10 +397,10 @@ void _TXSTA::start_transmitting()
 
 
   // Set a callback breakpoint at the next SPBRG edge
-
   if(cpu)
     get_cycles().set_break(spbrg->get_cpu_cycle(1), this);
 
+
   // The TSR now has data, so clear the Transmit Shift
   // Register Status bit.
 
@@ -400,10 +427,6 @@ void _TXSTA::transmit_break()
 
   bit_count = 14;  // 13 break, 1 stop
 
-  // Set a callback breakpoint at the next SPBRG edge
-  if(cpu)
-    get_cycles().set_break(spbrg->get_cpu_cycle(1), this);
-
   // The TSR now has data, so clear the Transmit Shift
   // Register Status bit.
 
@@ -413,6 +436,7 @@ void _TXSTA::transmit_break()
   // Tell the TXREG that its data has been consumed.
 
   mUSART->emptyTX();
+  callback();	// sent start bit
 }
 
 void _TXSTA::transmit_a_bit()
@@ -421,7 +445,7 @@ void _TXSTA::transmit_a_bit()
 
   if(bit_count) {
 
-    Dprintf(("Transmit bit #%x: bit val:%d time:0x%llx\n",bit_count, (tsr&1),get_cycles().get()));
+    Dprintf(("Transmit bit #%x: bit val:%d time:0x%"PRINTF_GINT64_MODIFIER"x\n",bit_count, (tsr&1), get_cycles().get()));
 
     putTXState(tsr&1 ? '1' : '0');
 
@@ -435,7 +459,7 @@ void _TXSTA::transmit_a_bit()
 
 void _TXSTA::callback()
 {
-  Dprintf(("TXSTA callback - time:%llx\n",get_cycles().get()));
+  Dprintf(("TXSTA callback - time:%"PRINTF_GINT64_MODIFIER"x\n",get_cycles().get()));
 
   transmit_a_bit();
 
@@ -619,7 +643,7 @@ void _RCSTA::receive_a_bit(unsigned int bit)
 
   // If we're waiting for the start bit and this isn't it then
   // we don't need to look any further
-  Dprintf(("receive_a_bit state:%d bit:%d time:0x%llx\n",state,bit,get_cycles().get()));
+  Dprintf(("receive_a_bit state:%d bit:%d time:0x%"PRINTF_GINT64_MODIFIER"x\n",state,bit,get_cycles().get()));
 
   if( state == RCSTA_MAYBE_START) {
     if (bit)
@@ -737,7 +761,7 @@ void _RCSTA::receive_start_bit()
 void _RCSTA::callback()
 {
 
-  Dprintf(("RCSTA callback. time:0x%llx\n",cycles.value));
+  Dprintf(("RCSTA callback. time:0x%"PRINTF_GINT64_MODIFIER"x\n",get_cycles().get()));
 
   // A bit is sampled 3 times.
 
@@ -874,7 +898,7 @@ void _SPBRG::get_next_cycle_break()
   if(cpu)
     get_cycles().set_break(future_cycle, this);
 
-  //Dprintf(("SPBRG::callback next break at 0x%llx\n",future_cycle));
+  //Dprintf(("SPBRG::callback next break at 0x%"PRINTF_GINT64_MODIFIER"x\n",future_cycle));
 
 }
 
@@ -914,14 +938,64 @@ unsigned int _SPBRG::get_cycles_per_tick()
 void _SPBRG::start()
 {
 
-  if(cpu)
-    last_cycle = get_cycles().get();
+  if(! skip  || get_cycles().get() >= skip) {
+    if(cpu)
+      last_cycle = get_cycles().get();
+    skip = 0;
+  }
+
   start_cycle = last_cycle;
 
   get_next_cycle_break();
 
-  Dprintf((" SPBRG::start   last_cycle:0x%llx: future_cycle:0x%llx\n",last_cycle,future_cycle));
+  Dprintf((" SPBRG::start   last_cycle:0x%"PRINTF_GINT64_MODIFIER"x: future_cycle:0x%"PRINTF_GINT64_MODIFIER"x\n",last_cycle,future_cycle));
+}
 
+void _SPBRG::put(unsigned int new_value)
+{
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
+
+  Dprintf((" SPBRG value=0x%x\n",value.get()));
+  //Prevent updating last_cycle until all current breakpoints have expired
+  //Otehrwise we see that rx/tx persiods get screwed up from now until future_cycle
+  future_cycle = last_cycle + get_cycles_per_tick();
+  skip = future_cycle;
+  Dprintf((" SPBRG value=0x%x skip=0x%"PRINTF_GINT64_MODIFIER"x last=0x%"PRINTF_GINT64_MODIFIER"x cycles/tick=0x%x\n",value.get(), skip, last_cycle, get_cycles_per_tick()));
+}
+
+void _SPBRG::put_value(unsigned int new_value)
+{
+
+  put(new_value);
+
+  update();
+
+}
+
+void _SPBRGH::put(unsigned int new_value)
+{
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
+
+  if(m_spbrg)
+    m_spbrg->set_start_cycle();
+}
+
+void _SPBRG::set_start_cycle()
+{
+  //Prevent updating last_cycle until all current breakpoints have expired
+  //Otherwise we see that rx/tx persiods get screwed up from now until future_cycle
+  future_cycle = last_cycle + get_cycles_per_tick();
+  skip = future_cycle;
+}
+
+void _SPBRGH::put_value(unsigned int new_value)
+{
+
+  put(new_value);
+
+  update();
 
 }
 
@@ -972,9 +1046,16 @@ guint64 _SPBRG::get_cpu_cycle(unsigned int edges_from_now)
 void _SPBRG::callback()
 {
 
-  last_cycle = get_cycles().get();
+  if (skip)
+  {
+  Dprintf((" SPBRG skip=0x%"PRINTF_GINT64_MODIFIER"x, cycle=0x%"PRINTF_GINT64_MODIFIER"x\n", skip, get_cycles().get()));
+  }
+  if(! skip  || get_cycles().get() >= skip) {
+    last_cycle = get_cycles().get();
+    skip = 0;
+  }
 
-  //Dprintf(("SPBRG rollover at cycle:0x%llx\n",last_cycle));
+  //Dprintf(("SPBRG rollover at cycle:0x%"PRINTF_GINT64_MODIFIER"x\n",last_cycle));
 
   if(rcsta && (rcsta->value.get() & _RCSTA::SPEN))
     {
