@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998,1999 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <iostream>
 #include <stdio.h>
@@ -41,6 +41,7 @@ class  _14bit_processor;
 
 class RXSignalSink;
 class TXSignalSource;
+class TXSignalControl;
 class USART_MODULE;
 
 
@@ -104,6 +105,7 @@ protected:
   USART_MODULE *mUSART;
   PinModule *m_PinModule;
   TXSignalSource *m_source;
+  TXSignalControl *m_control;
   char m_cTxState;
   bool bInvertPin;
 };
@@ -215,6 +217,7 @@ class _BAUDCON : public sfr_register
     WUE    = 1<<1,
     BRG16  = 1<<3,
     TXCKP  = 1<<4,
+    SCKP   = 1<<4,	// synchronous clock polarity Select bit (16f88x)
     RXDTP  = 1<<5,
     RCIDL  = 1<<6,
     ABDOVF = 1<<7
@@ -240,6 +243,8 @@ class _SPBRGH : public sfr_register
 
   _SPBRGH(Processor *pCpu, const char *pName, const char *pDesc);
   virtual void assign_spbrg(_SPBRG *new_spbrg) { m_spbrg = new_spbrg; };
+  virtual void put(unsigned int);
+  virtual void put_value(unsigned int);
  private:
   _SPBRG   *m_spbrg;
 };
@@ -267,8 +272,14 @@ class _SPBRG : public sfr_register, public TriggerObject
   virtual void get_next_cycle_break();
   virtual guint64 get_cpu_cycle(unsigned int edges_from_now);
   virtual guint64 get_last_cycle();
+
+  virtual void put(unsigned int);
+  virtual void put_value(unsigned int);
+  void set_start_cycle();
 // protected:
   virtual unsigned int get_cycles_per_tick();
+private:
+  guint64 skip;
 };
 
 //---------------------------------------------------------------
